# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import sys, traceback

try:
    import dbus
    import dbus.service
    import dbus.glib
    dbus.glib.threads_init()
    DBUS_AVAIL = True
except ImportError:
    # Dummy D-Bus library
    class _Connection:
        get_object = lambda *a: object()
    class _Interface:
        __init__ = lambda *a: None
        ListNames = lambda *a: []
    class Dummy: pass
    dbus = Dummy()
    dbus.Interface = _Interface
    dbus.service = Dummy()
    dbus.service.method = lambda *a: lambda f: f
    dbus.service.Object = object
    dbus.SessionBus = _Connection
    DBUS_AVAIL = False

import gobject
from optparse import OptionParser

options = None

class DBusInterfaceObject(dbus.service.Object):
    """
        A DBus service for exaile
    """
    def __init__(self, exaile, bus_name, path="/DBusInterfaceObject"):
        """
            Initializes the service
        """
        # HACK: Passing bus name as second argument is deprecated, but we need
        # it for compatibility with old versions of dbus-python, e.g. the one in
        # Ubuntu 6.10 (edgy).
        dbus.service.Object.__init__(self, bus_name, path)
        self.exaile = exaile

    @dbus.service.method("org.exaile.DBusInterface", "s")
    def play_file(self, filename):
        """
            Plays the specified file
        """
        print filename
        import common
        import xlmisc
        if common.any(filename.endswith(ext) for ext in xlmisc.PLAYLIST_EXTS):
            self.exaile.playlist_manager.import_playlist(filename, True)
        else:
            self.exaile.stream(filename)

    @dbus.service.method("org.exaile.DBusInterface", "s")
    def test_service(self, arg):
        """
            Just a method to test the service
        """
        print arg

    @dbus.service.method("org.exaile.DBusInterface")
    def prev_track(self):
        """
            Jumps to the previous track
        """
        self.exaile.player.previous()

    @dbus.service.method("org.exaile.DBusInterface")
    def stop(self):
        """
            Stops playback
        """
        self.exaile.player.stop()

    @dbus.service.method("org.exaile.DBusInterface")
    def next_track(self):
        """
            Jumps to the next track
        """
        self.exaile.player.next()

    @dbus.service.method("org.exaile.DBusInterface")
    def play(self):
        """
            Starts playback
        """
        self.exaile.player.play()

    @dbus.service.method("org.exaile.DBusInterface")
    def play_pause(self):
        """
            Toggle Play or Pause
        """
        self.exaile.player.toggle_pause()

    @dbus.service.method("org.exaile.DBusInterface", None, "s")
    def query(self):
        """
            Returns information about the currently playing track
        """
        if not self.exaile.player.current:
            return "No track playing"
        return self.exaile.player.current.full_status(self.exaile.player)

    @dbus.service.method("org.exaile.DBusInterface")
    def popup(self):
        """
            Shows a popup window with information about the current track
        """
        self.exaile.show_osd()

    @dbus.service.method("org.exaile.DBusInterface")
    def popup_text(self, text):
        """
            Shows a popup window with a text string
        """
        import xlmisc
        pop = xlmisc.get_osd(self, 
                xlmisc.get_osd_settings(self.exaile.settings))
        pop.show_osd(text, None)

    @dbus.service.method("org.exaile.DBusInterface", None, "s")
    def get_title(self):
        """
            Returns the title of the playing track
        """
        if not self.exaile.player.current:
            return ""
        return self.exaile.player.current.get_title()

    @dbus.service.method("org.exaile.DBusInterface", None, "s")
    def get_album(self):
        """
            Returns the album of the playing track
        """
        if not self.exaile.player.current:
            return ""
        return self.exaile.player.current.album

    @dbus.service.method("org.exaile.DBusInterface", None, "s")
    def get_artist(self):
        """
            Returns the artist of the playing track
        """
        if not self.exaile.player.current:
            return ""
        return self.exaile.player.current.artist

    @dbus.service.method("org.exaile.DBusInterface", None, "s")
    def get_length(self):
        """
            Returns the length of the playing track
        """
        if not self.exaile.player.current:
            return ""
        return self.exaile.player.current.length
    
    @dbus.service.method("org.exaile.DBusInterface", "s", "s")
    def get_track_attr(self, attr):
        """
            Attempts to return the specificed attribute of the 
            playing track, returns empty string on failure.
        """
        value = getattr(self.exaile.player.current, attr, None)
        if value:
            return unicode(value)
        return u''

    @dbus.service.method("org.exaile.DBusInterface", None, "y")
    def current_position(self):
        """
            Returns the position inside the current track as a percentage
        """
        if not self.exaile.player.current:
            return 0
        return self.exaile.player.get_current_position()

    @dbus.service.method("org.exaile.DBusInterface", None, "s")
    def status(self):
        """
            Returns if the player is paused or playing
        """
        return self.exaile.player.get_state()

    @dbus.service.method("org.exaile.DBusInterface", None, "s")
    def get_cover_path(self):
        """
            Returns the path to the cover image of the playing track
        """
        return self.exaile.cover.loc

    @dbus.service.method("org.exaile.DBusInterface", None, "s")
    def get_volume(self):
        return str(self.exaile.get_volume_percent())

    @dbus.service.method("org.exaile.DBusInterface", "y")
    def increase_volume(self,vol):
        """ 
            Increases the volume by vol
        """
        self.exaile.volume.increase(vol)

    @dbus.service.method("org.exaile.DBusInterface", "y")
    def decrease_volume(self,vol):
        """ 
            Decreases the volume by vol
        """
        self.exaile.volume.decrease(vol)

    @dbus.service.method("org.exaile.DBusInterface")
    def toggle_visibility(self):
        """
            Toggle the main window's visibility
        """
        if not self.exaile.window.get_property('visible'):
            self.exaile.window.present()
            self.exaile.setup_location()
        else:
           self.exaile.window.hide()

    @dbus.service.method("org.exaile.DBusInterface", None, "s")
    def get_version(self):
        return self.exaile.get_version()

    @dbus.service.method("org.exaile.DBusInterface", "y")
    def set_rating(self, rating):
        if not self.exaile.player.current: return
        self.exaile.set_rating(None, rating)

    @dbus.service.method("org.exaile.DBusInterface", None, "i")
    def get_rating(self):
        if not self.exaile.player.current: return -1
        try:
            return self.exaile.player.current._rating
        except ValueError:
            return 0

    @dbus.service.method("org.exaile.DBusInterface")
    def play_cd(self):
        self.exaile.open_disc()

def test_dbus(bus, interface):
    obj = bus.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
    dbus_iface = dbus.Interface(obj, 'org.freedesktop.DBus')
    avail = dbus_iface.ListNames()
    return interface in avail

def test(p):
    """
        Tests to see if the dbus service is running, and if it is, call
        methods on the service
    """

    global options
    (options, args) = p.parse_args()
    if not options.new:
        try:
            bus = dbus.SessionBus()
            if test_dbus(bus, 'org.exaile.DBusInterface'):
                remote_object = bus.get_object("org.exaile.DBusInterface",
                    "/DBusInterfaceObject")
                iface = dbus.Interface(remote_object, "org.exaile.DBusInterface")
                iface.test_service("testing dbus service")
                do_exit = False

                if options.get_title:
                    print iface.get_title()
                    do_exit = True
                if options.get_artist:
                    print iface.get_artist()
                    do_exit = True
                if options.get_album:
                    print iface.get_album()
                    do_exit = True
                if options.show_version:
                    print iface.get_version()
                    do_exit = True
                    sys.exit(0)
                if options.get_length:
                    print iface.get_length()
                    do_exit = True
                if options.current_position:
                    print iface.current_position()
                    do_exit = True

                if options.get_volume:
                    print iface.get_volume()                   
                    do_exit = True

                if options.get_rating:
                    print iface.get_rating()
                    do_exit = True

                if options.next: iface.next_track()
                elif options.prev: iface.prev_track()
                elif options.stop: iface.stop()
                elif options.play: iface.play()
                elif options.play_pause: iface.play_pause()
                elif options.guiquery: iface.popup()
                elif options.playcd: iface.play_cd()

                elif options.rating is not None:
                    iface.set_rating(options.rating)
                    do_exit = True
                elif options.inc_vol:
                    iface.increase_volume(options.inc_vol)
                elif options.dec_vol:
                    iface.decrease_volume(options.dec_vol)
                elif options.query:
                    print iface.query()

                elif len(sys.argv) > 1 and not sys.argv[1].startswith("-") and \
                    not do_exit:
                    iface.play_file(sys.argv[1])
                elif not do_exit:
                    iface.toggle_visibility()
                    print "You have entered an invalid option"

                return True
        except SystemExit:
            return True
        except:
            traceback.print_exc()
            True

    return False


def get_options():
    """
        Get the options for exaile
    """
    usage = "Usage: %prog [option...|uri]"
    p = OptionParser(usage=usage)
    p.add_option("-n", "--next", dest="next", action="store_true",
        default=False, help="Play the next track")
    p.add_option("-p", "--prev", dest="prev", action="store_true",
        default=False,   help="Play the previous track")
    p.add_option("-s", "--stop", dest="stop", action="store_true",
        default=False, help="Stop playback")
    p.add_option("-a", "--play", dest="play", action="store_true",
        default=False, help="Play")
    p.add_option("-t", "--play-pause", dest="play_pause", action="store_true",
        default=False, help="Toggle Play or Pause")
    p.add_option("-q", "--query", dest="query", action="store_true",
        default=False, help="Query player")
    p.add_option("--gui-query", dest="guiquery", action="store_true",
        default=False, help="Show a popup of the currently playing track")
    p.add_option("--get-title", dest="get_title", action="store_true",
        default=False, help="Print the title of current track")
    p.add_option("--get-album", dest="get_album", action="store_true",
        default=False, help="Print the album of current track")
    p.add_option("--get-artist", dest="get_artist", action="store_true",
        default=False, help="Print the artist of current track")
    p.add_option("--get-length", dest="get_length", action="store_true",
        default=False, help="Print the length of current track")
    p.add_option("--current-position", dest="current_position", action="store_true",
        default=False, help="Print the position inside the current track as a percentage")
    p.add_option("-i","--increase_vol", dest="inc_vol",action="store", 
        type="int",metavar="VOL",help="Increases the volume by VOL")
    p.add_option("-l","--decrease_vol", dest="dec_vol",action="store",
        type="int",metavar="VOL",help="Decreases the volume by VOL")
    p.add_option("--get-volume", dest="get_volume", action="store_true",
        default=False, help="Print the current volume")
    p.add_option("--play-cd", dest="playcd", action="store_true",
        default=False, help="Start playing a CD")
    p.add_option("--new", dest="new", action="store_true",
        default=False, help="Start new instance")
    p.add_option('--set-rating', dest='rating', help='Set rating for current '
        'song')
    p.add_option('--get-rating', dest='get_rating', help='Get rating for '
        'current song', default=False, action='store_true')
    p.add_option("--settings", dest="settings", help="Settings Directory")
    p.add_option("--cleanversion", dest="cleanversion", action="store_true")
    p.add_option("--version", dest="show_version", action="store_true")
    p.add_option("--testing", dest="testing", action="store_true")
    p.add_option("--no-equalizer", dest="noeq", action="store_true",
        default=False, help="Disable Equalizer support")
    p.add_option("--start-minimized", dest="minim", action="store_true",
        default=False, help="Start Exaile minimized to tray, if possible")

    return p



# encoding: utf-8

# Copyright (C) 2006 Adam Olsen 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from xl import xlmisc

try:
    from mutagen.mp4 import MP4 as MP4
except ImportError:
    from mutagen.m4a import M4A as MP4

TYPE = 'aac'

TAG_TRANSLATION = {
    '\xa9nam':      'title',
    '\xa9ART':      'artist',
    '\xa9alb':      'album',
    '\xa9gen':      'genre',
    '\xa9day':      'date',
    'trkn':         'tracknumber',
    'disk':         'discnumber',
    'cprt':         'copyright'
}

def get_tag(f, name):
    if not f.has_key(name):
        return [] 
    elif name in ['trkn', 'disk']: 
        ret = []
        for value in f[name]:
            ret.append("%d/%d" % (value[0], value[1]))
        return ret
    else: return [t for t in f[name]]

def set_tag(f, name, value):
    if type(value) is not list: value = [value]
    if name in ['trkn', 'disk']:
        try:
            f[name] = []
            for val in value:
                tmp = map(int, val.split('/'))
                f[name].append(tuple(tmp))
        except:
            xlmisc.log_exception()
    else:
        f[name] = value

def can_change(tag):
    return tag in TAG_TRANSLATION.values()

def is_multi():
    return True

def fill_tag_from_path(tr):
    try:
        f = MP4(tr.io_loc)
    except:
        xlmisc.log("Couldn't read tags from file: " + tr.loc)
        return

    tr.length = f.info.length
    tr.bitrate = f.info.bitrate
    
    for mp4_tag, tag in TAG_TRANSLATION.iteritems():
        try:
            tr.tags[tag] = get_tag(f, mp4_tag)
        except:
            tr.tags[tag] = [] 

def write_tag(tr):
    f = MP4(tr.io_loc)

    for mp4_tag, tag in TAG_TRANSLATION.iteritems():
        if tr.tags[tag]:
            set_tag(f, mp4_tag, tr.tags[tag])

    f.save()



#!/usr/bin/env python
# Copyright (C) 2007 Dan O'Reilly

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gobject
import gtk
from xl import common, media, library, plugins, xlmisc
from gettext import gettext as _
from xl.panels import collection
import xl.path


PLUGIN_NAME = "MTP Device Manager"
PLUGIN_AUTHORS = ["Dan O'Reilly <oreilldf@gmail.com>"]
PLUGIN_VERSION = "0.4.0"
PLUGIN_DESCRIPTION = _(r"""MTP device manager.
\n\nDepends:
\nlibmtp: http://libmtp.sourceforge.net/
\npymtp: http://nick125.com/projects/pymtp/""")
PLUGIN_ENABLED = False
PLUGIN_ICON = None

try:
    import pymtp
    MTP_INSTALLED = True
except:
    MTP_INSTALLED = False

CONNS = plugins.SignalContainer()


class MTPTrack(media.Track):
    '''
        MTPTrack class for holding media.Track info, the LIBMTP_Track
        item_id, and identifying the track type as 'device' so things get
        handled correctly by exaile.
    '''
    type = 'device'

    def __init__(self, title_t, artist_t, album_t, track_t, year_t, length_t, genre_t):
        '''
            Initializes a media.Track type with the necessary metadata
        '''
        media.Track.__init__(self, title=title_t, artist=artist_t, album=album_t,
                             genre=genre_t, track=track_t, year=year_t, length=length_t)
        self.type = 'device'
        mtp_item_id = None


class MTPPlaylist(object):
    """
        Container for MTP Playlist
    """

    def __init__(self, playlist, root=False):
        self.playlist = playlist
        self.root = root
        self.name = playlist.name
        self.tracks = playlist.tracks
        self.no_tracks = playlist.no_tracks
        self.playlist_id = playlist.playlist_id
    
    def __str__(self):
        return self.name


class MTPDeviceDriver(plugins.DeviceDriver):
    '''
        Class that implements an MTP Device Driver
    '''
    name = 'mtpdevice'
    
    def __init__(self):
        """
            Initializes the MTPDeviceDriver class.
        """
        plugins.DeviceDriver.__init__(self)
        self.connected = False
        self.connecting = False
        self.exaile = APP
        self.dp = APP.device_panel
        self.mtp_image = xlmisc.get_icon('gnome-dev-ipod')
        self.track_image = gtk.gdk.pixbuf_new_from_file(xl.path.get_data(
                                        'images', 'track.png'))
        self.playlist_image = gtk.gdk.pixbuf_new_from_file(xl.path.get_data(
                                        'images', 'playlist.png'))
        self.directory_image = xlmisc.get_icon('gnome-fs-directory')


    @common.threaded
    def connect(self, panel):
        '''
            Connects to the MTP device and loads playlist/track
             info into exaile.
        '''
        if self.connecting: self.disconnect()

        self.connecting = True
        self.panel = panel
        self.collection_panel = self.exaile.collection_panel
        self.mtp = pymtp.MTP()
        try:
            self.mtp.connect()
            self.connected = True
        except:
            self.panel.on_connect_complete(None)
            gobject.idle_add(panel.on_error, _("Couldn't connect to the MTP device,"
                                        " wait a few seconds or reconnect the"
                                        " device, then try again."))
            self.connected = False
            self.connecting = False
            return

        self.panel.track_count.set_label(_("Loading tracks..."))
        tracks = self.mtp.get_tracklisting()
        gobject.idle_add(self._load_tracks, tracks)
        
        self.panel.track_count.set_label(_("Loading playlists..."))
        self.playlists = {}
        for playlist in self.mtp.get_playlists():
            p_entry = MTPPlaylist(playlist)
            self.playlists[p_entry] = []
            for track in playlist.__iter__():
                self.playlists[p_entry].append(track)
            
        self.panel.track_count.set_label(_("Loading folders..."))

        self.connecting = False


    def _load_tracks(self, track_list):
        '''
            Loads the tracks from the connected device into the device panel
        '''
        self.all = library.TrackData()
        self.all_dict = {}
        for t in list(track_list):
            song = MTPTrack(t.title, t.artist, t.album, t.tracknumber,
                            t.date, t.duration/1000, t.genre)
            song.mtp_item_id = t.item_id
            # Since MTP tracks don't have an actual path, we use the
            # unique item_id the MTP player assigns it, so that exaile
            # has a way to hunt down the track from the panel tree.
            song.loc = str(t.item_id)
            gobject.idle_add(self.all.append, song)
            self.all_dict[t.item_id] = song
        self.connected = True
        gobject.idle_add(self.panel.on_connect_complete, self)


    def get_menu(self, item, menu):
        return menu


    def get_initial_root(self, model):
        """
            Adds new nodes and returns the one tracks should be added to
        """
        if not self.playlists: return None
        self.mtproot = model.append(None, [self.directory_image, _("Playlists"),
            'nofield'])

        for k, v in self.playlists.iteritems():
            item = model.append(self.mtproot, [self.playlist_image, k,
                                               'nofield'])
            self.load_playlist(item, v, model)
        new_root = model.append(None, [self.mtp_image, _("Music Collection"),
                                       'nofield'])
        return new_root
    
    
    def load_playlist(self, parent, list_contents, model):
        """
            Adds the track list for a given playlist to the device panel
            in Artst - Title format.
        """
        for track_id in list_contents:
            song = self.all_dict[track_id]
            if not song:
                print 'Couldn\'t find playlist track on device'
                continue
            name = song.artist + " - " + song.title
            model.append(parent, [self.track_image, name, 'nofield'])


    def done_loading_tree(self):
        """
            Called when the tree is done loading
        """
        path = self.dp.model.get_path(self.mtproot)
        self.dp.tree.expand_row(path, False)


    def disconnect(self, *a, **k):
        '''
            Disconnects the MTP device
        '''
        self.mtp.disconnect()
        self.panel = None
        self.connected = False
        self.connecting = False
        self.playlists = None
        self.all = None
        self.all_dict = None


    def remove_tracks(self, tracks):
        '''
            Removes tracks from the MTP device
        '''
        for track in tracks:
            print 'deleting ', track
            self.mtp.delete_object(int(track.mtp_item_id))
            self.all.remove(track)
            del self.all_dict[track.mtp_item_id]
            for playlist in self.playlists:
                try:
                    del playlist[track.mtp_item_id]
                except:
                    pass


    def search_tracks(self, keyword):
        '''
            Searches for a keyword if one is provided, and then sorts the
            results based on the panel drop-down box.
        '''
        if keyword:
            check = []
            for track in self.all:
                for item in ('artist', 'album', 'title'):
                    attr = getattr(track, item)
                    if keyword.lower() in attr.lower():
                        check.append(track)
        else:
            check = self.all

        new = []
        for song in check:
            a = {
                'album': library.lstrip_special(song.album),
                'artist': library.lstrip_special(library.the_cutter((song.artist))),
                'title': library.lstrip_special(song.title),
                'genre': library.lstrip_special(song.genre),
                'track': song.track,
            }

            if self.panel.choice.get_active() == 0:  # Artist view
                new.append((a['artist'], a['album'], a['track'], a['title'], song))
            elif self.panel.choice.get_active() == 1:  # Album view
                new.append((a['album'], a['artist'], a['track'], a['title'], song))
            else:  # Genre view
                new.append((a['genre'],a['artist'], a['album'], a['track'], a['title'], song))
        new.sort()
        return library.TrackData([a[-1] for a in new])


    def put_item(self, item):
        '''
            Transfers a track to the MTP device
        '''
        # Just for sanity
        track = item.track
        metadata = pymtp.LIBMTP_Track()

        # Load the metadata from the track into a LIBMTP_Track for transfer
        if (hasattr(track, 'artist')):
            metadata.artist = track.artist
        if (hasattr(track, 'title')):
            metadata.title = track.title
        if (hasattr(track, 'album')):
            metadata.album = track.album
        if (hasattr(track, 'track')):
            metadata.tracknumber = track.track
        if (hasattr(track, 'date')):
            metadata.date = track.date
        if (hasattr(track, 'genre')):
            metadata.genre = track.genre
        if (hasattr(track, '_len')):
            # Convert length to milliseconds so the MTP device
            # will display it properly
            metadata.duration = track.get_duration() * 1000

        song = MTPTrack(track.title, track.artist, track.album, metadata.tracknumber,
                        metadata.date, metadata.duration, metadata.genre)
        filename = str(track.get_filename())
        loc = str(track.get_loc())

        # Transfer the file
        track_id = self.mtp.send_track_from_file(loc, filename, metadata, parent=0)
        song.mtp_item_id = track_id #  Store the id for removing the track later
        # Assign the track a unique location
        song.loc = str(track_id)
        gobject.idle_add(self.all.append, song)
        self.all_dict[track_id] = song
    
    
    def get_song(self, song):
        """
            Transfers a track from the device and to the /tmp/ directory
            on the filesystem, and returns that track object.
        """
        new_loc = "/tmp/" + song.artist + '-' + song.title
        self.mtp.get_track_to_file(int(song.mtp_item_id), str(new_loc))
        song.loc = new_loc
        return song


def initialize():
    '''
        Initializes the plugin
    '''
    global PLUGIN
    
    if not MTP_INSTALLED:
        common.error(APP.window, _("The pymtp library could not be found.  Make"
                                   "sure you install both libmtp and pytmp "
                                   "(http://nick125.com/projects/pymtp/) before using this plugin."))
        return False
    PLUGIN = MTPDeviceDriver()
    if PLUGIN is None:
        return False
    APP.device_panel.add_driver(PLUGIN, PLUGIN_NAME)
    CONNS.connect(APP, 'quit', quit)
    return True


def destroy():
    '''
        Disconnects the device and removes the driver instance from the
        device panel.
    '''
    global PLUGIN
    if PLUGIN.connected is True:
        PLUGIN.mtp.disconnect()
    CONNS.disconnect_all()
    APP.device_panel.remove_driver(PLUGIN)
    PLUGIN = None


def quit(exaile):
    '''
        Disconnects the attached device when user quits exaile
    '''
    global PLUGIN
    if PLUGIN.connected is True:
        PLUGIN.mtp.disconnect()



# win32-mmkeys - Adds support for multimedia keys in MS Windows.
# Copyright (C) 2007 Johannes Sasongko <sasongko@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

PLUGIN_NAME = "Windows multimedia keys"
PLUGIN_AUTHORS = ["Johannes Sasongko <sasongko@gmail.com>"]

PLUGIN_VERSION = "0.1.1"
PLUGIN_DESCRIPTION = r"""Adds support for multimedia keys in MS Windows.
\n\nRequires pyHook, http://www.cs.unc.edu/Research/assist/developer.shtml"""
PLUGIN_ENABLED = False

import gtk
PLUGIN_ICON = gtk.Label().render_icon(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_MENU)


KEY_MAP = None # For fast key-to-function mapping.

def on_key_down(event):
    try:
        KEY_MAP[event.GetKey()]()
        return False # Swallow.
    except KeyError:
        return True


PLUGIN = None

def initialize():
    global KEY_MAP, PLUGIN

    KEY_MAP = {
        'Media_Prev_Track': APP.player.previous,
        'Media_Play_Pause': APP.player.toggle_pause,
        'Media_Stop': APP.player.stop,
        'Media_Next_Track': APP.player.next,
    }

    import pyHook
    PLUGIN = pyHook.HookManager()
    PLUGIN.KeyDown = on_key_down
    PLUGIN.HookKeyboard()

    return True

def destroy():
    global KEY_MAP, PLUGIN
    PLUGIN.UnhookKeyboard()
    KEY_MAP = PLUGIN = None



# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk, pynotify, traceback, cgi, os
from gettext import gettext as _
import xl.plugins as plugins
import xl.path

PLUGIN_NAME = _("LibNotify Plugin")
PLUGIN_AUTHORS = ['Adam Olsen <arolsen@gmail.com>']
PLUGIN_VERSION = '0.1.6'
PLUGIN_DESCRIPTION = _("Uses libnotify to inform you when a new song starts")
PLUGIN_ENABLED = False
button = gtk.Button()
PLUGIN_ICON = button.render_icon('gtk-info', gtk.ICON_SIZE_MENU)
button.destroy()
DEFAULT_SUMMARY = '{title}'
# TRANSLATORS: Libnotify plugin default message body
DEFAULT_BODY = _('{artist}\n<i>on {album}</i>')

GTKSTATUSICON_AVAIL = hasattr(gtk, 'StatusIcon')

APP = None
PLAY_ID = None
pynotify.init('exailenotify')

def configure():
    """
        Shows a configuration dialog that allows you to change the summary and
        body of the notification popup
    """
    exaile = APP
    settings = exaile.settings
    summary = settings.get_str('summary', default=DEFAULT_SUMMARY, plugin=plugins.name(__file__))
    body = settings.get_str('body', default=DEFAULT_BODY, plugin=plugins.name(__file__))

    dialog = plugins.PluginConfigDialog(exaile.window, PLUGIN_NAME)
    main = dialog.child
    label = gtk.Label(_("Notification Summary:"))
    label.set_alignment(0.0, 0.0)
    main.pack_start(label, False, False)

    summary_entry = gtk.Entry()
    summary_entry.set_text(summary)
    main.pack_start(summary_entry, True, False)

    label = gtk.Label(_("Notification Body:"))
    label.set_alignment(0.0, 0.0)
    main.pack_start(label, False, False)

    body_view = gtk.TextView()
    body_view.get_buffer().set_text(body)
    scroll = gtk.ScrolledWindow()
    scroll.add(body_view)
    scroll.set_shadow_type(gtk.SHADOW_IN)
    scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

    main.pack_start(scroll, True, True)

    show_cover_box = gtk.CheckButton(_('Show album covers in notification'))
    show_cover_box.set_active(settings.get_boolean('show_covers', default=True,
        plugin=plugins.name(__file__)))

    attach_to_tray_box = gtk.CheckButton(_('Attach notification to tray icon '
        '(if available)'))
    attach_to_tray_box.set_active(settings.get_boolean('attach_to_tray',
        default=True, plugin=plugins.name(__file__)))
    main.pack_start(attach_to_tray_box)


    main.pack_start(show_cover_box)
    dialog.resize(280, 240)
    dialog.show_all()

    result = dialog.run()
    dialog.hide()
    if result == gtk.RESPONSE_OK:   
        buf = body_view.get_buffer()
        start = buf.get_start_iter()
        end = buf.get_end_iter()
        settings.set_str('body', buf.get_text(start, end), plugin=plugins.name(__file__))
        settings.set_str('summary', summary_entry.get_text(), plugin=plugins.name(__file__))
        settings.set_boolean('show_covers', show_cover_box.get_active(),
            plugin=plugins.name(__file__))
        settings.set_boolean('attach_to_tray',
            attach_to_tray_box.get_active(), plugin=plugins.name(__file__))

def play_track(exaile, track):
    """
        Called when a track starts playing.
        Displays a notification via notification daemon
    """
    settings = APP.settings

    vals = dict()
    vals['summary'] = settings.get_str('summary', default=DEFAULT_SUMMARY, plugin=plugins.name(__file__))
    vals['body'] = settings.get_str('body', default=DEFAULT_BODY, plugin=plugins.name(__file__))

    for k, val in vals.iteritems():
        for item in ('title', 'artist', 'album', 'length', 'track', 'bitrate',
            'genre', 'year', 'rating'):
            try:
                value = getattr(track, item)
                if type(value) != str and type(value) != unicode:
                    value = unicode(value)

                # escape html entities
                value = cgi.escape(value)

                vals[k] = vals[k].replace("{%s}" % item, value)
            except AttributeError:
                trackback.print_exc()

    notify = pynotify.Notification(vals['summary'], vals['body'])

    if settings.get_boolean('show_covers', default=True,
        plugin=plugins.name(__file__)):
        pixbuf = gtk.gdk.pixbuf_new_from_file(APP.cover_manager.fetch_cover(track, 1))
        pixbuf = pixbuf.scale_simple(50, 50, gtk.gdk.INTERP_BILINEAR)
    else:
        pixbuf = gtk.gdk.pixbuf_new_from_file(xl.path.get_data('images', 'largeicon.png'))
        pixbuf = pixbuf.scale_simple(50, 50, gtk.gdk.INTERP_BILINEAR)

    notify.set_icon_from_pixbuf(pixbuf)

    attach_to_tray = settings.get_boolean('attach_to_tray', default=True,
        plugin=plugins.name(__file__))
    if attach_to_tray and APP.tray_icon:
        icon = APP.tray_icon.icon
        if isinstance(icon, gtk.Widget):
            notify.attach_to_widget(icon)
        elif GTKSTATUSICON_AVAIL and isinstance(icon, gtk.StatusIcon):
            notify.set_property('status-icon', icon)

    notify.show()

def initialize():
    """
        Initializes the plugin. 
        In this plugin, not much needs to be done except for set up the
        globals
    """
    global PLAY_ID
    PLAY_ID = APP.player.connect('play-track', play_track)
    return True

def destroy():
    """
        No cleanup needs to be done for this plugin
    """
    global PLAY_ID
    if PLAY_ID:
        APP.disconnect(PLAY_ID)
        PLAY_ID = None



#!/usr/bin/env python

import sys, os

plugindir = sys.argv[1]
cwd = os.getcwd()

userdir = os.path.expanduser('~/.exaile/plugins')
os.chdir('plugins/%s' % plugindir)
os.unlink(os.path.join(userdir, '%s.exz' % plugindir))
os.system('zip -r %s/%s.exz *' % (userdir, plugindir))
os.chdir(cwd)
os.system("./exaile")



# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
from gettext import gettext as _
from xl import common, xlmisc

class LibraryDialog(object):
    """
        Allows you to choose which directories are in your library
    """
    def __init__(self, exaile):
        """
            Initializes the dialog
        """
        self.exaile = exaile
        self.xml = gtk.glade.XML('exaile.glade', 'LibraryManager', 'exaile')
        self.dialog = self.xml.get_widget('LibraryManager')
        self.list = xlmisc.ListBox(self.xml.get_widget('lm_list_box'))
        self.add_list = []
        self.remove_list = []
        self.dialog.set_transient_for(exaile.window)
        self.xml.get_widget('lm_add_button').connect('clicked',
            self.on_add)
        self.xml.get_widget('lm_remove_button').connect('clicked',
            self.on_remove)

        self.xml.get_widget('lm_cancel_button').connect('clicked',
            lambda e: self.dialog.response(gtk.RESPONSE_CANCEL))
        self.xml.get_widget('lm_apply_button').connect('clicked',
            self.on_apply)

        import_location = self.exaile.settings.get_str('import/location', '')
        lm_import_dir = self.xml.get_widget('lm_import_dir')
        if import_location:
            lm_import_dir.set_text(_('Import Directory: %s') % import_location)
        else:
            lm_import_dir.hide()

        items = exaile.settings.get_list("search_paths", [])
        self.items = []
        for i in items:
            if i: self.items.append(i)

        if self.items:
            self.list.set_rows(self.items)

    def run(self):
        """
            Runs the dialog, waiting for a response before any other gui
            events occur
        """
        return self.dialog.run()

    def get_response(self):
        """
            Gets the response id
        """
        return self.dialog.get_response()

    def destroy(self):
        """
            Destroys the dialog
        """
        self.dialog.destroy()

    def get_items(self):
        """
            Returns the items in the dialog
        """
        return self.items

    def on_apply(self, widget):
        """
            Saves the paths in the dialog, and updates the library
        """
        self.exaile.settings['search_paths'] = self.list.rows
        self.exaile.settings['add_paths'] = self.add_list
        self.exaile.settings['remove_paths'] = self.remove_list
        self.dialog.response(gtk.RESPONSE_APPLY)

    def on_remove(self, widget):
        """
            removes a path from the list
        """
        item = self.list.get_selection()
        if item is None:
            return
        index = self.list.rows.index(item)
        
        import_dir = self.exaile.settings.get_str('import/location', '')
        scan_import_dir = self.exaile.settings.get_boolean(\
            'import/scan_import_dir', True)

        if import_dir and not scan_import_dir:
            # if scan_import_dir is false, it means that the import dir doesn't
            # appear in this list at all, it is encompassed by another dir.
            # so we check that, and re-add the import dir if necessary.
            if import_dir.startswith(item):
                self.list.append(import_dir)
                self.add_list.append(import_dir)
                self.exaile.settings['import/scan_import_dir'] = True
        
        self.remove_list.append(item)
        self.list.remove(item)
        selection = self.list.list.get_selection()
        if index > len(self.list.rows):
            selection.select_path(index - 1)
        else:
            selection.select_path(index)

    def on_add(self, widget):
        """
            Adds a path to the list
        """
        dialog = gtk.FileChooserDialog(_("Add a directory"),
            self.exaile.window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_ADD, gtk.RESPONSE_OK))
        dialog.set_current_folder(self.exaile.get_last_dir())
        response = dialog.run()
        dialog.hide()

        if response == gtk.RESPONSE_OK:
            path = dialog.get_filename()
            import_location = self.exaile.settings.get_str('import/location')
            tmp_items = self.items + [import_location]

            for item in tmp_items:
                if not item: continue
                if path.startswith(item):
                    # our added path is a subdir of an existing path
                    common.error(self.exaile.window, _('Path is already '
                        'in your collection, or is a subdirectory of '
                        'another path in your collection'))
                    return
                elif item.startswith(path):
                    # path is a parent to some previously added directories
                    if item == import_location:
                        # we don't remove the import dir from the list.
                        # instead we don't scan it anymore, since the
                        # dir we are adding right now covers it
                        self.exaile.settings['import/scan_import_dir'] = False
                    else:
                        xlmisc.log('Library Manager: Newly added directory '
                            'contains the directory %s, which will be '
                            'removed from the list' % item)
                        self.list.remove(item)
    
            self.add_list.append(path)
            self.list.append(path)

        dialog.destroy()



# Copyright (C) 2006 Adam Olsen 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import fileinput, xlmisc, re

class ConvertIniToConf:

    def __init__(self, settings, loc):
        self.settings = settings
        self.loc = loc
        self.osettings = {}

        try:
            for line in fileinput.input(loc):
                line = line.strip()
                try:
                    (key, value) = line.split(" = ")
                    self.osettings[key] = value
                except ValueError:
                    pass
        except:
            xlmisc.log("Error reading settings file, using defaults.")

        old_map = {
            "tab_placement": ("ui/tab_placement", self.conv_int),
            "use_popup": ("osd/enabled", self.conv_bool),
            "last_active": self.conv_int,
            "show_track_col_Album": ("ui/track_columns", self.add_rem_list),
            "open_last": self.conv_bool,
            "Artist_trackcol_width": self.conv_width,
            "show_track_col_Rating": ("ui/track_columns", self.add_rem_list),
            "save_queue": self.conv_bool,
            "trackslist_defaults_set": ("ui/trackslist_defaults_set", self.conv_bool),
            "use_tray": ("ui/use_tray", self.conv_bool),
            "show_track_col_Length": ("ui/track_columns", self.add_rem_list),
            "watch_exclude_dirs": self.conv_watch_exclude,
            "show_track_col_Genre": ("ui/track_columns", self.add_rem_list),
            "show_track_col_#": ("ui/track_columns", self.add_rem_list),
            "show_track_col_Title": ("ui/track_columns", self.add_rem_list),
            "osd_h": ("osd/h", self.conv_int),
            "osd_bgcolor": ("osd/bgcolor", self.conv_str),
            "audio_sink": self.conv_str,
            "mainw_width": ("ui/mainw_width", self.conv_int),
            "search_paths": ("search_paths", self.conv_co_list),
            "Length_trackcol_width": self.conv_width,
            "mainw_height": ("ui/mainw_height", self.conv_int),
            "osd_x": ("osd/x", self.conv_int),
            "osd_y": ("osd/y", self.conv_int),
            "col_active_view": ("ui/col_active_view", self.conv_int),
            "show_track_col_Artist": ("ui/track_columns", self.add_rem_list),
            "osd_textcolor": ("osd/text_color", self.conv_str),
            "Title_trackcol_width": self.conv_width,
            "amazon_locale": self.conv_str,
            "osd_w": ("osd/w", self.conv_int),
            "fetch_covers": self.conv_bool,
            "Album_trackcol_width": self.conv_width,
            "use_splash": ("ui/use_splash", self.conv_bool),
            "art_filenames": ("art_filenames", self.conv_ss_list),
            "lastfm_user": ("lastfm/user", self.conv_str),
            "files_panel_dir": self.conv_str,
            "osd_tray": ("osd/tray", self.conv_bool),
            "active_view": ("ui/active_view", self.conv_int),
            "#_trackcol_width": self.conv_width,
            "repeat": self.conv_bool,
            "osd_display_text": ("osd/display_text", self.conv_str),
            "ensure_visible": ("ui/ensure_visible", self.conv_bool),
            "mainw_sash_pos": ("ui/mainw_sash_pos", self.conv_int),
            "osd_text_font": ("osd/text_font", self.conv_str),
            "wikipedia_locale": self.conv_str,
            "cd_device": self.conv_str,
            "scan_interval": self.conv_float,
            "lastfm_pass": ("lastfm/pass", self.conv_str),
            "show_track_col_Bitrate": ("ui/track_columns", self.add_rem_list),
            "watch_directories": self.conv_bool,
            "show_track_col_Year": ("ui/track_columns", self.add_rem_list),
            "mainw_x": ("ui/mainw_x", self.conv_int),
            "mainw_y": ("ui/mainw_y", self.conv_int),
	    "podcast_show_limit": self.conv_str
        }

        for setting in self.osettings.keys():
            if old_map.has_key(setting):
                v = old_map[setting]
                if isinstance(v, tuple):
                    v[1](setting, v[0])
                else:
                    v(setting, setting)
            elif re.match("^[A-Za-z_]+\.py.*$", setting):
                self.conv_plugin(setting)

    def conv_int(self, old_key, new_key):
        self.settings.set_int(new_key, int(self.osettings[old_key]))
    
    def conv_float(self, old_key, new_key):
        self.settings.set_float(new_key, float(self.osettings[old_key]))

    def conv_str(self, old_key, new_key):
        self.settings.set_str(new_key, str(self.osettings[old_key].replace(r"\n", '\n')))

    def conv_bool(self, old_key, new_key):
        self.settings.set_boolean(new_key, self.osettings[old_key])

    def conv_width(self, old_key, old_key_again):
        name, new_key_prefix = old_key.split("_", 1)
        self.settings.set_int("ui/%s_%s" % (new_key_prefix, name), int(self.osettings[old_key]))

    def add_rem_list(self, old_key, new_key):
        values = self.settings.get_list(new_key)

        col_name = old_key.rsplit("_", 1)[1]

        if self.osettings[old_key] == 'true' or self.osettings[old_key] == 'True':
            if col_name not in values:
                values.append(col_name)
                self.settings.set_list(new_key, values)
        else:
            if col_name in values:
                values.remove(col_name)
                self.settings.set_list(new_key, values)

    def conv_ss_list(self, old_key, new_key):
        values = self.osettings[old_key].split(" ")
        self.settings.set_list(new_key, values)

    def conv_co_list(self, old_key, new_key):
        values = self.osettings[old_key].split(":")
        self.settings.set_list(new_key, values)

    def conv_watch_exclude(self, old_key, new_key):
        values = self.osettings[old_key]
        if values == "incomplete":
            self.settings.set_list(new_key, [])
        else:
            self.conv_ss_list(old_key, new_key)

    def conv_plugin(self, old_key):
        py_ext = old_key.find('.py')
        key_name = old_key[py_ext+4:]
        plugin_name = old_key[:py_ext]

        if key_name == "plugin_enabled":
            self.settings.set_boolean("enabled", self.osettings[old_key], plugin=plugin_name)
        else:
            value = self.osettings[old_key]

            if re.match("^\d+$", value):
                self.settings.set_int(key_name, int(value), plugin=plugin_name)
            elif re.match("^\d*\.\d+$", value):
                self.settings.set_float(key_name, float(value), plugin=plugin_name)
            elif re.match("^[Tt]rue|[Ff]alse$", value):
                self.settings.set_boolean(key_name, value, plugin=plugin_name)
            else:
                self.settings.set_str(key_name, value, plugin=plugin_name)



#!/usr/bin/env python
import gtk, gobject, time
from gettext import gettext as _
import xl.plugins as plugins

PLUGIN_NAME = _("Resume Playback")
PLUGIN_AUTHORS = ['Jonas Wagner <veers' + chr(32+32) + 'gmx' + '.ch>']
PLUGIN_VERSION = "0.2.3"
PLUGIN_DESCRIPTION = _(r"""Resumes playback after restarting exaile""")

PLUGIN_ENABLED = False
PLUGIN_ICON = None
PLUGIN = None
SETTINGS = None
TIMER_ID = None
PLAYING = "playing"
TRACK = "track"
PROGRESS = "progress"

def restore_state():
    """
    Tries to restore the saved state form the settings
    """
    while gtk.events_pending():
        gtk.main_iteration()
    if APP.settings.get_boolean(PLAYING, default=False, plugin=PLUGIN_NAME):
        loc = APP.settings.get_str(TRACK, plugin=PLUGIN_NAME)
        track = APP.all_songs.for_path(loc)
        
        if not track: return
        track.submitted = True
        APP.player.play_track(track)
        APP.player.current = track
        APP.player.seek(APP.settings.get_float(PROGRESS, default=0.1, plugin=PLUGIN_NAME))

def save_state(sender):
    """
    save the current song and playback status to the config
    """
    track = APP.player.current
    if track:
        APP.settings.set_boolean(PLAYING, APP.player.is_playing(), plugin=PLUGIN_NAME)
        APP.settings.set_str(TRACK, track.io_loc, plugin=PLUGIN_NAME)
        APP.settings.set_float(PROGRESS, APP.player.get_current_position() * track.duration /\
                100.0, plugin=PLUGIN_NAME)
    else:
        APP.settings.set_boolean(PLAYING, False, plugin=PLUGIN_NAME)

def initialize():
    """
    Connect to the PluginEvents
    """
    APP.connect("quit", save_state)
    APP.playlist_manager.connect("last-playlist-loaded", 
            lambda sender: gobject.idle_add(restore_state))
    return True

def destroy():
    """
    Do nothing    
    """
    pass



#!/usr/bin/env python

import os
from distutils.core import setup, Extension

def capture(cmd): return os.popen(cmd).read().strip()

setup(name = "pymmkeys", version = "2",
      url = "http://sacredchao.net/~piman/software/python.shtml",
      description = "multimedia key input for PyGTK",
      author = "Joe Wreschnig",
      author_email = "piman@sacredchao.net",
      license = "GNU GPL v2",
      long_description = """
This module lets you access multimedia keys found on most new keyboards
from PyGTK; most importantly it grabs all input events so your program
doesn't need to be in focus when the key is pressed (which is the
usual behavior of the keys).""",
      ext_modules=[
         Extension("mmkeys",
                   ["mmkeyspy.c", "mmkeys.c", "mmkeysmodule.c"],
                   extra_compile_args = capture("pkg-config --cflags gtk+-2.0 pygtk-2.0").split(),
                   extra_link_args = capture("pkg-config --libs gtk+-2.0 pygtk-2.0").split()
                   ),
         ])



import mutagen, mutagen.id3, mutagen.mp3
from xl import xlmisc

TYPE = 'mp3'
IDS = { 
        "TIT2": "title",
        "TIT3": "version",
        "TPE1": "artist",
        "TPE2": "performer",
        "TPE3": "conductor",
        "TPE4": "arranger",
        "TEXT": "lyricist",
        "TCOM": "composer",
        "TCON": "genre",
        "TENC": "encodedby",
        "TALB": "album",
        "TRCK": "tracknumber",
        "TPOS": "discnumber",
        "TSRC": "isrc",
        "TCOP": "copyright",
        "TPUB": "organization",
        "TSST": "part",
        "TOLY": "author",
        "TBPM": "bpm",
        "TDRC": "date",
        "TDOR": "originaldate",
        "TOAL": "originalalbum",
        "TOPE": "originalartist",
        "WOAR": "website",
        }

def get_tag(id3, t):
    """
        Reads a specific id3 tag from the file
    """
    if not id3.has_key(t): return [] 
    field = id3.getall(t)

    ret = []
    if t == 'TDRC' or t == 'TDOR': # values are ID3TimeStamps
        for value in field:
            ret.extend([unicode(x) for x in value.text])
    else:
        for value in field:
            try:
                ret.extend([unicode(x.replace('\n','').replace('\r','')) \
                    for x in value.text])
            except:
                xlmisc.log("Can't parse ID3 field")
                xlmisc.log_exception()
    return ret

def write_tag(tr):
    try:
        id3 = mutagen.id3.ID3(tr.io_loc)
    except mutagen.id3.ID3NoHeaderError:
        id3 = mutagen.id3.ID3()

    for id3name, key in IDS.items():
        id3.delall(id3name)

    for k, v in IDS.iteritems():
        if tr.tags[v]:
            try:
                frame = mutagen.id3.Frames[k](encoding=3,
                    text = tr.tags[v])
                id3.loaded_frame(frame)
            except:
                xlmisc.log_exception()

    id3.save(tr.io_loc)    

def can_change(tag):
    return tag in IDS.values()

def is_multi():
    return True

def fill_tag_from_path(tr):
    try:
        info = mutagen.mp3.MP3(tr.io_loc)
    except:
        xlmisc.log("Couldn't read tags from file: " + tr.loc)
        return

    tr.length = info.info.length
    tr.bitrate = info.info.bitrate

    try:    
        id3 = mutagen.id3.ID3(tr.io_loc)

        for id3_tag, tag in IDS.iteritems():
            tr.tags[tag] = get_tag(id3, id3_tag)
            
    except OverflowError:
        pass
    except mutagen.id3.ID3NoHeaderError:
        pass
    except IOError:
        pass
    except:
        xlmisc.log_exception()



# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import pygtk
pygtk.require('2.0')

import gtk, gobject, os, signal, sys, thread, pango, gtk.glade
import xl.dbusinterface, time, datetime
from gettext import gettext as _
from xl import library, logger, media, equalizer, burn, common
from xl import xlmisc, config, db, covers, player, prefs
from xl import playlist as playlistmanager
from xl.plugins import manager as pluginmanager, gui as plugingui
from xl.gui import playlist as trackslist
from xl.gui import information, tray
from xl.panels import collection, radio, playlists, files, device
import random, gst, urllib
from lib import scrobbler
try:
    import cPickle as pickle
except ImportError:
    import pickle

def found_updates(exaile, found):
    message = _("The following plugins have new versions available for install."
    " You can install them from the plugin manager.\n\n")

    for (name, version) in found:
        message += "%s\t%s\n" % (name, version)

    dialog = gtk.MessageDialog(exaile.window, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO,
        gtk.BUTTONS_OK)
    dialog.set_markup(message)
    dialog.add_button(_("Plugin Manager"), gtk.RESPONSE_YES)
    result = dialog.run()
    dialog.destroy()
    if result == gtk.RESPONSE_YES:
        manager = exaile.show_plugin_manager() 
        manager.plugin_nb.set_current_page(2)

@common.threaded
def start_updatecheck_thread(playlist_manager):
    exaile = playlist_manager.exaile
    # check exaile itself
    version = map(int, exaile.get_version().replace('devel', 
        '').replace('b', '').split('.'))
    check_version = map(int,
        urllib.urlopen('http://exaile.org/current_version.txt').read().replace('devel', 
        '').replace('b', '').split('.'))

    if version < check_version:
        gobject.idle_add(common.info, exaile.window, _("Exaile version %s is "
            "available.  Grab it from http://www.exaile.org today!") % 
            '.'.join([str(i) for i in check_version]))

    # check plugins
    pmanager = exaile.pmanager
    avail_url = 'http://www.exaile.org/files/plugins/%s/plugin_info.txt' % \
            exaile.get_plugin_location()

    h = urllib.urlopen(avail_url)
    lines = h.readlines()
    h.close()

    found = []

    check = False
    for line in lines:
        line = line.strip()
        (file, name, version, author, description) = line.split('\t')
        
        for plugin in pmanager.plugins:
            if plugin.PLUGIN_NAME == name:
                installed_ver = map(int, plugin.PLUGIN_VERSION.split('.'))
                available_ver = map(int, version.split('.'))

                if installed_ver < available_ver:
                    found.append((name, version))

    if found:
        gobject.idle_add(found_updates, exaile, found)

class ExaileWindow(gobject.GObject): 
    """
        The main interface class
    """
    __gsignals__ = {
        'seek': (gobject.SIGNAL_RUN_LAST, None, (int,)),
        'quit': (gobject.SIGNAL_RUN_LAST, None, ()),
        'tray-icon-toggled': (gobject.SIGNAL_RUN_LAST, None, (bool,)),

        # called when the title label is changed (sometimes it changes when
        # the track hasn't changed, for example when you're listening to a
        # shoutcast stream)
        'track-information-updated': (gobject.SIGNAL_RUN_LAST, None, ()),
        'quit': (gobject.SIGNAL_RUN_LAST, None, ()),
        'timer_update': (gobject.SIGNAL_RUN_LAST, None, ()),
        'lastfm_toggle': (gobject.SIGNAL_RUN_LAST, None, (bool,))
    }
    __single = None

    # ExaileWindow should be a singleton
    @classmethod
    def get_instance(cls, options, first_run=False):
        """
            Use this to get the Exaile instance
        """
        if not ExaileWindow.__single:
            instance = ExaileWindow(options, first_run)
            ExaileWindow.__single = instance

        return ExaileWindow.__single

    def __init__(self, options, first_run=False): 
        """
            Initializes the main Exaile window
        """
        if ExaileWindow.__single:
            raise AssertionError, 'Exaile instance has already been created'

        gobject.GObject.__init__(self)
        self.xml = gtk.glade.XML(xlmisc.glade_file(self), 'ExaileWindow', 'exaile')
        self.window = self.xml.get_widget('ExaileWindow')
        media.exaile_instance = self

        self.settings = config.Config(xl.path.get_config('settings.ini'))

        self.options = options
        config.settings = self.settings
        self.database_connect()
        self.all_songs = library.TrackData()
        self.songs = library.TrackData()
        self.playlist_songs = library.TrackData()
        self.library_manager = library.LibraryManager(self)
        self.tracks = None
        self.playlists_menu = None
        self.playlist_manager = playlistmanager.PlaylistManager(self)
        self.timer = xlmisc.MiscTimer(self.timer_update, 1000)
        self.cover_manager = covers.CoverManager(self)
        self.plugin_tracks = {}
        self.playing = False
        self.thread_pool = []
        self.dir_queue = []
        self.scan_timer = None
        self.seeking = False
        self.debug_dialog = logger.init(self, xl.path.get_config('exaile.log'))
        self.col_menus = dict()
        self.setup_col_menus('track', trackslist.TracksListCtrl.COLUMNS)
        self.plugins_menu = xlmisc.Menu()
        self.rewind_track = 0
        self.player = player.ExailePlayer(self)
        self.submit_time_played = 0
        self.submit_time = 0
        self.player.tag_func = self.tag_callback
        self.importer = xl.cd_import.CDImporter(self)
        self.audio_disc_page = None
        self.urlhandlers = []

        # check for updates
        if self.settings.get_boolean('check_for_updates', True):
            self.playlist_manager.connect('last-playlist-loaded',
                start_updatecheck_thread)

        if self.settings.get_boolean("ui/use_splash", True):
            image = gtk.Image()
            image.set_from_file(xl.path.get_data('images', 'splash.png'))

            xml = gtk.glade.XML('exaile.glade', 'SplashScreen', 'exaile')
            splash_screen = xml.get_widget('SplashScreen')
            box = xml.get_widget('splash_box')
            box.pack_start(image, True, True)
            splash_screen.set_transient_for(None)
            splash_screen.show_all()
            xlmisc.finish()
            gobject.timeout_add(2500, splash_screen.destroy) 
        
        # connect to dbus
        if not "win" in sys.platform:
            import dbus
            try:
                conn = dbus.SessionBus()
                name = dbus.service.BusName("org.exaile.DBusInterface", conn)
                self.dbus_object = xl.dbusinterface.DBusInterfaceObject(
                    self, name)
            except dbus.DBusException:
                xlmisc.log("Could not connect to dbus session bus.  "
                    "dbus will be unavailable.")

        self.tray_icon = None

        self.volume = xlmisc.Adjustment(0, 0, 100, 1, 10, 0)
        self.volume.connect('value-changed', self.__on_volume_changed)
        self.volume.set_value(self.settings.get_float('volume', .7) * 100)

        vol = self.xml.get_widget('volume_slider')
        vol.set_adjustment(self.volume)
        vol.connect('scroll-event', self.__on_volume_scroll)
        vol.connect('key-press-event', self.__on_volume_key_press)

        if self.settings.get_boolean("ui/use_tray", False): 
            self.setup_tray()
    
        # TRANSLATORS: The title of the main window
        self.window.set_title(_("Exaile Music Player"))

        self.playlists_nb = self.xml.get_widget('playlists_nb')
        self.set_tab_placement()
        self.setup_left()
        self.setup_right()
        self.connect_events()
        self.setup_menus()

        pos = self.settings.get_int("ui/mainw_sash_pos", 200)
        self.setup_location()

        self.splitter = self.xml.get_widget('splitter')
        self.splitter.connect('notify::position', self.on_resize)
        self.splitter.set_position(pos)

        self.status = xlmisc.StatusBar(self)

        # log in to audio scrobbler
        user = self.settings.get_str("lastfm/user", "")
        password = self.settings.get_crypted("lastfm/pass", "")

        self.player.connect('play-track', self.as_play_track)
        self.player.connect('stop-track', self.as_stop_track)
        if user and password:
            self.scrobbler_login(user, password)
            
        # Try to load a saved last.fm cache
        self.scrobbler_load_cache()
        
        self.playlists_nb.connect('switch-page', self.page_changed)
        try:
            self.playlists_nb.connect('page-added', self.sync_playlists_tabbar)
            self.playlists_nb.connect('page-removed', self.sync_playlists_tabbar)
        except TypeError:
            # Pre-GTK+ 2.10
            xlmisc.log("Using old GtkNotebook")
        self.playlists_nb.remove_page(0)

        self.timer.start()

        self.window.show_all()

        if not self.settings.get_boolean('ui/show_stop_button', True):
            self.stop_button.hide()
        if not self.settings.get_boolean('ui/show_clear_button', True):
            self.clear_button.hide()
        if not self.settings.get_boolean('ui/show_cover', True):
            self.xml.get_widget('main_cover_frame').hide()

        self.stop_track_button.set_sensitive(False)
        self.pmanager = pluginmanager.Manager(self, self.update_plugin) 
        enabled_plugins = []
        for k, v in self.settings.get_plugins().iteritems():
            if v:
                if k.endswith(".exz"):
                    enabled_plugins.append(k.replace('.exz', '.py'))
                else:
                    enabled_plugins.append("%s.py" % k)

        self.pmanager.load_plugins(xl.path.get_config('plugins'),
            enabled_plugins)
        self.library_manager.load_songs(False, True)

        if first_run:
            dialog = gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,
                gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
                _("You have not specified any search directories for your "
                "music library. You may do so now, or choose to do it later.  "
                "If you want to do it later, you can manage your library "
                "search directories by going to Edit->Library Manager.  "
                "Do you want to choose your library directories now?")) 
            result = dialog.run()
            dialog.destroy()
            if result == gtk.RESPONSE_YES:
                self.library_manager.show_library_manager()

        interval = self.settings.get_float('scan_interval', 25)
        if interval:
            self.start_scan_interval(interval)

        # Handle SIGTERM
        signal.signal(signal.SIGTERM, self.on_sigterm)

    def tag_callback(self, tags):
        """
            Called when a tag is found in a stream
        """
        track = self.player.current
        if not (track and track.type == 'stream'): return True

        log = ['Stream tag:']
        newsong=False

        for key in tags.keys():
            value = tags[key]
            try:
                value = common.to_unicode(value)
            except UnicodeDecodeError:
                log.append('  ' + key + " [can't decode]: " + `str(value)`)
                continue # TODO: What encoding does gst give us?

            log.append('  ' + key + ': ' + value)

            if key == 'bitrate': track.bitrate = int(value) / 1000

            # if there's a comment, but no album, set album to the comment
            elif key == 'comment' and not track.loc.endswith('.mp3'): track.album = value
            elif key == 'album': track.album = value
            elif key == 'artist': track.artist = value
            elif key == 'duratin': track._len = long(value)
            elif key == 'track-number': 
                try:
                    track.track = int(value)
                except ValueError: pass
            elif key == 'genre': track.genre = value
            elif key == 'title': 
                try:
                    if track.rawtitle != value:
                        track.rawtitle = value
                        newsong = True
                except AttributeError:
                    track.rawtitle = value
                    newsong = True

                title_array = value.split(' - ', 1)
                if len(title_array) == 1 or (track.loc.endswith(".mp3") and \
                    not track.loc.endswith("lastfm.mp3")):
                    track.title = value
                else:
                    track.artist = title_array[0]
                    track.title = title_array[1]

        self.tracks.refresh_row(track)
        self.update_track_information()
        if newsong:
            log.append('  New song, fetching cover.')
            self.cover_manager.fetch_cover(track)
            self.show_osd()

        xlmisc.log_multi(log)
        return True

    def get_version(self):
        """
            Returns the version of Exaile
        """
        return sys.modules['__main__'].__version__

    def get_plugin_location(self):
        """
            Returns the location of the plugins
        """
        if sys.modules['__main__'].__version__.find('devel') > -1 \
            or sys.modules['__main__'].__version__.find('b') > -1:
            return 'trunk'
        else:
            return sys.modules['__main__'].__version__

    def start_scan_interval(self, value):
        """
            Starts the scan timer with the specified value in minutes, or 0 to
            disable
        """
        if not value:
            if self.scan_timer:
                self.scan_timer.stop()
                self.scan_timer = None
            xlmisc.log("Scan timer is disabled.")
            return

        if not self.scan_timer:
            self.scan_timer = xlmisc.MiscTimer(lambda:
                self.library_manager.on_library_rescan(load_tree=False), 1) 

        xlmisc.log('Starting scan timer at %s' % value)
        self.scan_timer.stop()
        self.scan_timer.time = int(value * 60 * 1000)
        self.scan_timer.start()

    def setup_location(self):
        """
            Sets up the location and size of the window based on settings
        """
        if self.settings.get_boolean('ui/mainw_maximized', False):
            self.window.maximize()

        width = self.settings.get_int("ui/mainw_width", 640)
        height = self.settings.get_int("ui/mainw_height", 475)

        x = self.settings.get_int("ui/mainw_x", 10)
        y = self.settings.get_int("ui/mainw_y", 10)

        self.window.resize(width, height)
        self.window.move(x, y)

    def setup_col_menus(self, pref, cols):
        """
            Fetches the view column menus from the glade xml definition
        """
        self.resizable_cols = self.xml.get_widget('cols_resizable_item')
        self.not_resizable_cols = \
            self.xml.get_widget('cols_not_resizable_item')
        self.resizable_cols.set_active(self.settings.get_boolean('ui/resizable_cols',
            False))
        self.not_resizable_cols.set_active(not self.settings.get_boolean('ui/resizable_cols',
            False))
        self.resizable_cols.connect('activate', self.activate_cols_resizable)
        self.not_resizable_cols.connect('activate',
            self.activate_cols_resizable)

        column_ids = None
        if self.settings.get_boolean('ui/new_trackslist_defaults_set', False):
            column_ids = set()
            ids = self.settings.get_list("ui/%s_columns" % pref)
            # Don't add invalid columns.
            all_ids = frozenset(trackslist.TracksListCtrl.COLUMN_IDS)
            for id in ids:
                if id in all_ids:
                    column_ids.add(id)

        if not column_ids:
            # Use default.
            ids = trackslist.TracksListCtrl.default_column_ids
            self.settings.set_boolean('ui/new_trackslist_defaults_set', True)
            self.settings.set_list('ui/%s_columns' % pref, ids)
            column_ids = frozenset(ids)

        self.col_menus[pref] = {}

        for col_struct in cols:
            self.col_menus[col_struct.id] = menu = self.xml.get_widget(
                '%s_%s_col' % (pref, col_struct.id))

            menu.set_active(col_struct.id in column_ids)
            menu.connect('activate', self.change_column_settings,
                ('ui/%s_columns' % pref, col_struct))

    def activate_cols_resizable(self, widget, event=None):
        """
            Called when the user chooses whether or not columns can be
            resizable
        """
        self.settings.set_boolean('ui/resizable_cols',
            self.resizable_cols.get_active())
        for i in range(0, self.playlists_nb.get_n_pages()):
            page = self.playlists_nb.get_nth_page(i)
            if isinstance(page, trackslist.TracksListCtrl):
                page.update_col_settings()

    def change_column_settings(self, item, data):
        """
            Changes column view settings
        """
        pref, col_struct = data
        id = col_struct.id

        column_ids = list(self.settings.get_list(pref))
        if item.get_active():
            if id not in column_ids:
                xlmisc.log("adding %s column to %s" % (id, pref))
                column_ids.append(id)
        else:
            if col_struct.id in column_ids:
                xlmisc.log("removing %s column from %s" % (id, pref))
                column_ids.remove(id)
        self.settings.set_list(pref, column_ids)

        for i in range(0, self.playlists_nb.get_n_pages()):
            page = self.playlists_nb.get_nth_page(i)
            if isinstance(page, trackslist.TracksListCtrl):
                page.update_col_settings()

    def page_changed(self, nb, page, num):
        """
            Called when the user switches pages
        """
        page = nb.get_nth_page(num)
        if isinstance(page, trackslist.TracksListCtrl):
            if isinstance(page, trackslist.QueueManager): return
            self.tracks = page
            self.update_songs(page.songs, False)

    def queue_count_clicked(self, *e):
        """
            Called when the user clicks the queue count label
        """
        if self.queue_count_label.get_label(): 
            self.show_queue_manager()

    def connect_events(self):
        """
            Connects events to the various widgets
        """
        self.window.connect('configure_event', self.on_resize)
        self.window.connect('window_state_event', self.on_state_change)
        self.window.connect('delete_event', self.on_quit)
        self.queue_count_label = self.xml.get_widget('queue_count_label')
        self.xml.get_widget('queue_count_box').connect('button-release-event',
            self.queue_count_clicked)

        # for multimedia keys
        self.mmkeys = xlmisc.MmKeys('Exaile', self.__on_mmkey)
        keygrabber = self.mmkeys.grab()
        xlmisc.log("Using multimedia keys from: " + str(keygrabber))

        self.play_button = self.xml.get_widget('play_button')
        self.play_button.connect('clicked', lambda *e: self.player.toggle_pause())

        self.stop_button = self.xml.get_widget('stop_button')
        self.stop_button.connect('clicked', lambda *e: self.player.stop())

        self.xml.get_widget('randomize_item').connect('activate', lambda *e:
            self.randomize_playlist())

        self.xml.get_widget('show_visualizations_item').connect('activate', 
            lambda *e: player.show_visualizations(self))

        self.quit_item = self.xml.get_widget('quit_item')
        self.quit_item.connect('activate', self.on_quit)

        self.plugins_item = self.xml.get_widget('plugins_item')
        self.plugins_item.connect('activate', self.show_plugin_manager)
        self.view_menu = self.xml.get_widget('view_menu')

        self.new_progressbar = self.xml.get_widget('new_progressbar')
        self.new_progressbar.set_fraction(0)
        # TRANSLATORS: Progress bar background text when there is no playback
        self.new_progressbar.set_text(_("Not Playing"))
        self.new_progressbar.connect('button-press-event', self.seek_begin)
        self.new_progressbar.connect('button-release-event', self.seek_end)
        self.new_progressbar.connect('motion-notify-event',
            self.seek_motion_notify)

        self.clear_button = self.xml.get_widget('clear_button')
        self.clear_button.connect('clicked', lambda *e: self.clear_playlist(None))

        self.next_button = self.xml.get_widget('next_button')
        self.next_button.connect('clicked', lambda e: self.player.next())

        self.previous_button = self.xml.get_widget('prev_button')
        self.previous_button.connect('clicked', lambda *e: self.player.previous())

        self.tracks_filter = xlmisc.ClearEntry(self.live_search)
        self.xml.get_widget('tracks_filter_box').pack_start(
            self.tracks_filter.entry,
            True, True)
        self.tracks_filter.connect('activate', self.on_search)
        self.key_id = None

        self.rescan_collection = self.xml.get_widget('rescan_collection')
        self.rescan_collection.connect('activate', 
            self.library_manager.on_library_rescan)

        self.library_item = self.xml.get_widget('library_manager')
        self.library_item.connect('activate', lambda e:
            self.library_manager.show_library_manager())

        self.equalizer_item = self.xml.get_widget('equalizer_item')
        self.equalizer_item.connect('activate', lambda e:
            self.show_equalizer())

        self.queue_manager_item = self.xml.get_widget('queue_manager_item')
        self.queue_manager_item.connect('activate', 
            lambda *e: self.show_queue_manager())

        self.blacklist_item = self.xml.get_widget('blacklist_manager_item')
        self.blacklist_item.connect('activate', lambda e:
            self.show_blacklist_manager())

        self.xml.get_widget('clear_button').connect('clicked',
            self.clear_playlist)
        self.accel_group = gtk.AccelGroup()
        key, mod = gtk.accelerator_parse('<Control>C')
        self.accel_group.connect_group(key, mod, gtk.ACCEL_VISIBLE,
            self.clear_playlist)

        key, mod = gtk.accelerator_parse('<Control>W')
        self.accel_group.connect_group(key, mod, gtk.ACCEL_VISIBLE,
            lambda *e: self.close_page())

        key, mod = gtk.accelerator_parse('<Control>L')
        self.accel_group.connect_group(key, mod, gtk.ACCEL_VISIBLE,
            lambda *e: self.jump_to(3))

        # toggle last.fm submissions
        key, mod = gtk.accelerator_parse('<Control><Shift>L')
        self.accel_group.connect_group(key, mod, gtk.ACCEL_VISIBLE,
            lambda *e: self.toggle_lastfm())
        
        # add the accel group to the window
        self.window.add_accel_group(self.accel_group)

        self.xml.get_widget('preferences_item').connect('activate',
            lambda e: prefs.Preferences(self).run())

        self.clear_queue_item = self.xml.get_widget('clear_queue_item')

        self.goto_current_item = self.xml.get_widget('goto_current_item')
        self.goto_current_item.connect('activate', self.goto_current)

        self.xml.get_widget('about_item').connect('activate',
            lambda *e: xlmisc.show_about_dialog(self.window,
            self.get_version_info()))
        
        self.xml.get_widget('new_item').connect('activate',
            lambda *e: self.new_page())

        self.open_item = self.xml.get_widget('open_item')
        self.open_item.connect('activate', self.on_add_media)

        self.xml.get_widget('export_playlist_item').connect('activate',
            lambda *e: self.playlist_manager.export_playlist())

        self.xml.get_widget('open_url_item').connect('activate',
            self.open_url)

        self.fetch_item = self.xml.get_widget('fetch_covers_item')
        self.fetch_item.connect('activate',
            self.cover_manager.fetch_covers)


        self.open_disc_item = self.xml.get_widget('open_disc_item')
        self.open_disc_item.connect('activate',
            self.open_disc)

        self.xml.get_widget('track_information_item').connect('activate',
            lambda *e: self.jump_to(0))

        action_log_item = self.xml.get_widget('action_log_item')
        action_log_item.connect('activate',
            lambda *e: self.show_debug_dialog()) 

        self.xml.get_widget('import_directory_item').connect('activate',
            lambda *e: self.library_manager.import_directory(load_tree=True))

        self.rating_combo = self.xml.get_widget('rating_combo')
        self.rating_combo.set_active(0)
        self.rating_combo.set_sensitive(False)
        self.rating_signal = self.rating_combo.connect('changed', self.set_rating)

        # stop track event box
        self.stop_track_button = self.xml.get_widget('stop_track_button')
        self.stop_track_button.connect('clicked',
            self.stop_track_toggle)

    def toggle_lastfm(self):
        """
            Toggles last.fm submissions
        """
        new_setting = not self.settings.get_boolean('lastfm/submit', True)
        self.settings.set_boolean('lastfm/submit', new_setting)
        xlmisc.log("Toggling last.fm submissions to: %s" % new_setting)
        self.emit('lastfm_toggle', new_setting)

        type = _('Enabling')
        if not new_setting: type = _('Disabling')
        message = type + ' ' + _('Last.FM Submissions...')
        self.status.set_first(message, 2000)

    def get_version_info(self):
        """
            Gets the current version information
            If this is a development version, add on the revision number.
            This number is in xl/version.py, which is generated when the
            "make" command is issued (via bzr version-info --format=python >
            xl/version.py
        """
        version = sys.modules['__main__'].__version__
        if 'devel' in version:
            import xl.version
            version = "%s [r%s]" % (version, xl.version.version_info['revno'])

        return version

    def stop_track_toggle(self, *e):
        """
            Stop track toggle
        """
        if not self.player.stop_track: return
        result = common.yes_no_dialog(self.window, 
            _('Playback is currently set to stop on '
            'the track "%(title)s" by "%(artist)s". '
            'Would you like to remove this?') % {
                'title': self.player.stop_track.title,
                'artist': self.player.stop_track.artist
            })
        if result == gtk.RESPONSE_YES:
            self.player.stop_track = None
            self.tracks.queue_draw()

    def __on_mmkey(self, key):
        if key in ('Play', 'PlayPause', 'Pause'):
            self.player.toggle_pause()
        elif key == 'Stop':
            self.player.stop(1)
        elif key == 'Previous':
            self.player.previous()
        elif key == 'Next':
            self.player.next()

    def randomize_playlist(self):
        """
            Randomizes the current playlist
        """
        songs = self.tracks.songs
        random.shuffle(songs)
        self.tracks.set_songs(songs)

    def show_plugin_manager(self, *e):
        """
            Shows the plugin manager
        """
        manager = plugingui.PluginManager(self, self.window, self.pmanager,
            self.update_plugin,
            'http://www.exaile.org/files/plugins/%s/plugin_info.txt' %
            self.get_plugin_location())
        return manager

    def update_plugin(self, plugin):
        """
            Sets whether or not a plugin is enabled
        """
        self.settings.set_boolean("enabled", plugin.PLUGIN_ENABLED, plugin=plugin.FILE_NAME)

    def set_rating(self, combo=None, rating=None):
        """
            Sets the user rating of a track
        """
        track = self.player.current
        if not track: return

        if rating is None:
            rating = combo.get_active()
        else:
            try:
                rating = int(rating)
            except ValueError:
                xlmisc.log('Invalid rating passed')
                return
            if rating < 0: rating = 0
            if rating > 5: rating = 5
        track.rating = rating
        path_id = library.get_column_id(self.db, 'paths', 'name', track.loc)
        self.db.execute("UPDATE tracks SET user_rating=? WHERE path=?", 
            (rating, path_id))

        xlmisc.log("Set rating to %d for track %s" % (rating, track))
        if self.tracks:
            self.tracks.refresh_row(track)


    def show_debug_dialog(self):
        """
            Shows the debug dialog if it has been initialized
        """
        if logger.gui:
            logger.gui.dialog.show()

    def live_search(self, *e):
        """
            Simulates live search of tracks
        """
        if self.key_id:
            gobject.source_remove(self.key_id)

        self.key_id = gobject.timeout_add(700, self.on_search, None, None,
            False)

    def on_clear_queue(self, *e):
        """
            Called when someone wants to clear the queue
        """
        self.player.queued = []
        if not self.tracks: return
        self.tracks.queue_draw()

    def show_queue_manager(self):
        """
            Shows the queue manager
        """
        nb = self.playlists_nb
        for i in range(0, nb.get_n_pages()):
            page = nb.get_nth_page(i)
            if page.type == 'queue':
                nb.set_current_page(i)
                return
        page = trackslist.QueueManager(self)
        # TRANSLATORS: Title of the Queue tab
        tab = xlmisc.NotebookTab(self, _("Queue"), page)
        self.playlists_nb.append_page(page, tab)
        self.playlists_nb.set_tab_reorderable(page, True)
        self.playlists_nb.set_current_page(
            self.playlists_nb.get_n_pages() - 1)

        # if there is more than one tab, show the tab bar
        if self.playlists_nb.get_n_pages() > 1:
            self.playlists_nb.set_show_tabs(True)

    def show_blacklist_manager(self, new=True):
        """
            Shows the blacklist manager
        """
        nb = self.playlists_nb
        all = self.db.select("""
            SELECT 
                paths.name, 
                path 
            FROM 
               tracks, paths 
            WHERE 
                paths.id=tracks.path AND 
                blacklisted=1 
            ORDER BY 
                artist, album, track, title
        """)
        songs = []
        for row in all:
            song = library.read_track(self.db, None, row[0])
            if song: songs.append(song)

        for i in range(0, nb.get_n_pages()):
            page = nb.get_nth_page(i)
            if page.type == 'blacklist':
                nb.set_current_page(i)
                page.set_songs(songs)
                return
        if not new: return

        page = trackslist.BlacklistedTracksList(self)
        page.set_songs(songs)
        # TRANSLATORS: Title of the Blacklist tab
        tab = xlmisc.NotebookTab(self, _("Blacklist"), page)
        self.playlists_nb.append_page(page, tab)
        self.playlists_nb.set_current_page(
            self.playlists_nb.get_n_pages() - 1)
        self.playlists_nb.set_tab_reorderable(page, True)
        # if there is more than one tab, show the tab bar
        if self.playlists_nb.get_n_pages() > 1:
            self.playlists_nb.set_show_tabs(True)

    def show_equalizer(self):

        try: # Equalizer element is still not very common 
            gst.element_factory_make('equalizer-10bands')
        except gst.PluginNotFoundError: # Should probably log this..
            common.error(self.window, _('GStreamer equalizer is not '
                'available.  It can be found in gstreamer-plugins-bad 0.10.5.'))
            return
        eq = equalizer.EqualizerWindow(self)

    def get_play_image(self, size=gtk.ICON_SIZE_SMALL_TOOLBAR):
        """
            Returns a play image
        """
        return gtk.image_new_from_stock('gtk-media-play', size)

    def get_pause_image(self, size=gtk.ICON_SIZE_SMALL_TOOLBAR):
        """
            Returns a pause image
        """
        return gtk.image_new_from_stock('gtk-media-pause', size)

    def set_tab_placement(self, setting=None):
        """
            Sets the placement of the tabs on the playlists notebook
        """
        if not setting:
            p = self.settings.get_int('ui/tab_placement', 0)
        else: p = setting
        s = gtk.POS_LEFT
        if p == 0: s = gtk.POS_TOP
        elif p == 1: s = gtk.POS_LEFT
        elif p == 2: s = gtk.POS_RIGHT
        elif p == 3: s = gtk.POS_BOTTOM

        self.playlists_nb.set_show_border(True)
        self.playlists_nb.set_tab_pos(s)
        
    def setup_tray(self): 
        """
            Sets up the tray icon
        """
        if not tray.USE_TRAY:
            xlmisc.log("Sorry, tray icon is NOT available")
            return
        if self.tray_icon: return
        self.tray_icon = tray.TrayIcon(self)
        self.emit('tray-icon-toggled', True)

    def remove_tray(self):
        """
            Removes the tray icon
        """
        if self.tray_icon:
            self.emit('tray-icon-toggled', False)
            self.tray_icon.destroy()
            self.tray_icon = None

    def _load_tab(self, last_active):
        """
            Selects the last loaded page
        """
        xlmisc.finish()
        xlmisc.log('Loading page %s' % last_active)
        self.playlists_nb.set_current_page(last_active)
        page = self.playlists_nb.get_nth_page(last_active)
        self.tracks = page
        if not page: return
        self.update_songs(page.songs, False)



    def on_blacklist(self, item, event):
        """
            Blacklists tracks (they will not be added to the library on
            collection scan
        """
        if not self.tracks: return
        result = common.yes_no_dialog(self.window, _("Blacklisting the selected "
            "tracks will prevent them from being added to the library on"
            " rescan.  Are you sure you want to continue?"))
        if result == gtk.RESPONSE_YES:
            self.tracks.delete_tracks(None, 'blacklist')

    def on_dequeue(self, item, param): 
        """
            Dequeues the selected tracks
        """
        tracks = self.tracks.get_selected_tracks()
        for track in tracks:
            try: self.player.queued.remove(track)
            except ValueError: pass
            
        self.tracks.queue_draw()
        trackslist.update_queued(self)

    def on_queue(self, item, param, toggle=True): 
        """
            Queues the selected tracks to be played after the current lineup
        """
        songs = self.tracks.get_selected_tracks()

        first = True
        for track in songs:
            if track in self.player.queued:
                if toggle:
                    self.player.queued.remove(track)

            elif first and track == self.player and self.player.is_playing():
                pass
            else:
                self.player.queued.append(track)

            first = False
        
        self.tracks.queue_draw()
        trackslist.update_queued(self)
        
    def on_stop_track(self, item, param, toggle=True): 
        """
            Stops playback after the selected track
        """
        track = self.tracks.get_selected_track()

        if self.player.stop_track == track:
            self.player.stop_track = None
        else:
            self.player.stop_track = track

    def setup_left(self): 
        """
            Sets up the left panel
        """
        self.panel_names = {}
        self.panel_widgets = {}

        self.playlists_panel = playlists.PlaylistsPanel(self)
        self.collection_panel = collection.CollectionPanel(self)
        self.side_notebook = self.xml.get_widget('side_notebook')
        self.files_panel = files.FilesPanel(self)

        page_number = self._find_page_number('device_box')
        self.device_panel = device.DevicePanel(self)
        self.device_panel_widget = self.side_notebook.get_nth_page(page_number)
        self.device_panel_label = self.side_notebook.get_tab_label(
            self.device_panel_widget)

        self.side_notebook.remove_page(page_number)
        self.device_panel_showing = False

        self.pradio_panel = radio.RadioPanel(self)

        for panel in ('col', 'playlists', 'files', 'radio'):
            if not self.settings.get_boolean('ui/show_%s_panel' % panel, True):
                self.set_panel_visible(panel, False)

    def set_panel_visible(self, name, show):
        """
            Shows or hides a panel
        """
        page_number = self._find_page_number('%s_box' % name)
        if not show:
            self.panel_widgets[name] = \
                self.side_notebook.get_nth_page(page_number)
            self.panel_names[name] = self.side_notebook.get_tab_label(
                self.panel_widgets[name])
            self.side_notebook.remove_page(page_number)
        else:
            if not self.panel_widgets.has_key(name): return
            self.side_notebook.append_page(self.panel_widgets[name],
                self.panel_names[name])

    def _find_page_number(self, text):
        """
            Finds a specific page number for a label
        """
        for i in range(self.side_notebook.get_n_pages()):
            page = self.side_notebook.get_nth_page(i)
            if page.get_name() == text: return i

        return 0

    def show_device_panel(self, show):
        """
            Toggles whether or not the device panel is showing
        """
        if not self.device_panel_showing and show:
            self.side_notebook.append_page(self.device_panel_widget,
                self.device_panel_label)
        elif self.device_panel_showing and not show:
            self.side_notebook.remove_page(
                self.side_notebook.page_num(self.device_panel_widget))

        self.device_panel_showing = show

    def get_database(self):
        """
            Returns a new database connection
        """
        loc = xl.path.get_config('music.db')
        database = db.DBManager(loc)
        database.add_function_create(('THE_CUTTER', 1, library.the_cutter))
        database.add_function_create(('LSTRIP_SPEC', 1, library.lstrip_special))
        return database

    def database_connect(self):
        """
            Connects to the database
        """

        im = False
        if not os.path.isfile(xl.path.get_config('music.db')):
            im = True
        try:
            self.db = self.get_database()
        except db.DBOperationalError, e:
            common.error(self.window, _("Error connecting to database: %s" % e))
            sys.exit(1)
        if im:
            try:
                self.db.import_sql(xl.path.get_data('sql', 'db.sql'))
            except db.DBOperationalError, e:
                common.error(self.window, _("Error "
                    "creating collection database: %s") % e)
                sys.exit(1)

        # here we check for the "version" table.  If it's there, it's an old
        # style (0.2.6) database, so we upgrade it
        else:
            try:
                cur = self.db.realcursor()
                cur.execute('SELECT version FROM version')
                cur.close()
                self.db = db.convert_to027(self.db.db_loc)
                self.db.add_function_create(('THE_CUTTER', 1, 
                    library.the_cutter))
            except:
                pass # db is ok, continue!

        self.db.check_version(xl.path.get_data('sql'))

    def add_urlhandler(self, handler):
        """
            Items in the urlhandlers list will be queried for certain patterns
            to see if they can handle playback of these items, IE, lastfm://
            would be handled by the lastfmproxy plugin
        """
        if not handler in self.urlhandlers:
            self.urlhandlers.append(handler)

    def remove_urlhandler(self, handler):
        """
            Removes a handler from the system
        """
        if handler in self.urlhandlers:
            self.urlhandlers.remove(handler)

    def initialize(self):
        """
            Called when everything is done loading
        """
        xlmisc.finish()
        self.playlist_manager.load_last_playlist()

        if len(sys.argv) > 1 and sys.argv[1] and \
            not sys.argv[1].startswith("-"):
            self.stream(sys.argv[1])
        if self.options.playcd:
            self.open_disc()
        if self.options.minim:
            try:
                self.tray_icon.toggle_exaile_visibility()
            except:
                pass
        return False

    @common.threaded
    def update_songs(self, songs=None, set=True): 
        """
            Sets the songs and playlist songs
        """
        tracks = self.tracks
        if not tracks:
            tracks = self.playlists_nb.get_nth_page(0)
        if not songs and tracks: songs = tracks.songs
        self.songs = songs
        self.playlist_songs = songs


        if set: 
            try:
                visible = self.tracks.list.get_visible_range()
            except AttributeError:
                # compatibility with old gtk versions (e.g. in Ubuntu Dapper)
                # that don't have TreeView.get_visible_range
                visible = None
            if visible: 
                (path1, path2) = visible

                scroll_to_end = False
                if path2 and path2[0] == len(self.tracks.songs):
                    scroll_to_end = True

            gobject.idle_add(tracks.set_songs, songs)

            if visible and path1 and path2:
                if scroll_to_end:
                    gobject.idle_add(tracks.list.scroll_to_cell,
                    (len(self.tracks.songs),))
                else:
                    gobject.idle_add(tracks.list.scroll_to_cell, path1[0]+1)

    def timer_update(self, event=None): 
        """
            Fired every half second.
            Updates the seeker position, the "now playing" title, and
            submits the track to last.fm when appropriate
        """
        status_text = ""
        track_count = len(self.songs)

        if track_count:
            #TRANSLATORS: Number of tracks in the playlist
            status_text += _("%d showing") % track_count

            total_time = self.songs.get_total_length()
            if total_time:
                status_text += " (" + total_time + ")"

            status_text += ", "

        #TRANSLATORS: Number of tracks in the collection
        status_text += _("%d in collection") % len(self.all_songs)
        self.status.set_track_count(status_text)

        track = self.player.current

        self.rewind_track += 1

        if track is None: 
            return True
        duration = track.duration

        # update the progress bar/label
        value = self.player.get_current_position()
        if duration == -1:
            real = 0
        else:
            real = value * duration / 100
        seconds = real

        if not self.seeking and not self.player.is_paused():
            self.new_progressbar = self.xml.get_widget('new_progressbar')
            fraction = value / 100
            if fraction > 1: fraction = 1
            self.new_progressbar.set_fraction(fraction)

            if track.type == 'stream':
                if track.start_time and self.player.is_playing():
                    seconds = time.time() - track.start_time
                    self.new_progressbar.set_text("%d:%02d" % # TODO: i18n
                        (seconds // 60, seconds % 60))

            else:
                remaining_seconds = duration - seconds
                self.new_progressbar.set_text("%d:%02d / %d:%02d" % # TODO: i18n
                    (seconds // 60, seconds % 60,
                    remaining_seconds // 60, remaining_seconds % 60))

        if not self.player.is_paused():
            self.submit_time_played += 1

        self.emit('timer_update')

        return True

    def as_play_track(self, player, track):
        """
            Called when playback of a track has started
        """
        self.submit_time = int(time.mktime(datetime.datetime.now().timetuple()))
        self.submit_time_played = 0
        scrobbler.now_playing(track.artist, track.title, track.album,
            int(track.duration), track.track)

    def as_stop_track(self, player, track):
        """
            Called when playback of a track has stopped
        """
        if not track: return
        if track.type == 'podcast' or track.type == 'stream': return

        if self.submit_time_played > 240 or float(self.submit_time_played) > \
            float(track.duration) / 2.0:
            self.update_rating(track, plays=1, rating=1)

            # check to see if this track should be submitted to audioscrobbler
            if self.settings.get_boolean('lastfm/submit', True) \
                and track.duration > 30:

                xlmisc.log("[Last.FM]: Attempting to submit %s to last.fm" %
                    track)
                self.submit_to_scrobbler(track)
                self.submit_time_played = 0

    def scrobbler_load_cache(self):
        """
            Loads the audioscrobbler cache, if it exists
        """
        cache = xl.path.get_config('lastfm.db')
        if os.path.isfile(cache):
            h = open(cache, 'rb')
            cache_data = pickle.load(h)
            h.close()
            scrobbler.SUBMIT_CACHE = cache_data

            os.remove(cache)

    def scrobbler_write_cache(self):
        """
            Saves scrobbler cache data
        """
        if scrobbler.SUBMIT_CACHE:
            cache = xl.path.get_config('lastfm.db')
            h = file(cache, 'wb')
            pickle.dump(scrobbler.SUBMIT_CACHE, h, 2)
            h.close()

    @common.threaded
    def scrobbler_login(self, user, password):
        """
            Logs in to audioscrobbler
        """
        try:
            scrobbler.login(user, password, client=('exa', self.get_version()),
                hashpw=True)
        except:
            xlmisc.log("Error logging into audioscrobbler")
            xlmisc.log_exception()

        try:
            if scrobbler.SUBMIT_CACHE:
                xlmisc.log("[Last.FM]: Attempting to flush %d item(s) "
                    "from scrobbler cache" % len(scrobbler.SUBMIT_CACHE))
                scrobbler.flush()
        except:
            xlmisc.log_exception()

    @common.threaded
    def submit_to_scrobbler(self, track):
        """
            Submits a track to audioscrobbler
        """
        if scrobbler.SESSION_ID:
            try:
                scrobbler.submit(track.artist, track.title, self.submit_time, 'P',
                    '', int(track.duration), track.album, track.track, autoflush=True)
            except:
                gobject.idle_add(self.status.set_first, _("Error submitting "
                    "to audioscrobbler"), 3000)
        track.submitted = True

    def update_track_information(self, track='', returntrue=True):
        """
            Updates track status information
        """
        self.rating_combo.disconnect(self.rating_signal)
        if track == '':
            track = self.player.current

        self.artist_label = self.xml.get_widget('artist_label')
        if track == None:
            self.new_progressbar.set_fraction(0)
            self.new_progressbar.set_text(_("Not Playing"))
            self.title_label.set_label(_("Not Playing"))
            self.artist_label.set_label(_("Stopped"))
            self.rating_combo.set_active(0)
            self.rating_combo.set_sensitive(False)

            self.rating_signal = self.rating_combo.connect('changed',
                self.set_rating)
            return

        title = track.title
        album = track.album
        artist = track.artist

        if artist:
            # TRANSLATORS: Window title
            self.window.set_title(_("%(title)s (by %(artist)s)") %
                { 'title': title, 'artist': artist } + " - Exaile")
        else:
            self.window.set_title(title + " - Exaile")

        self.title_label.set_label(title)

        if album or artist:
            desc = []
            # TRANSLATORS: Part of the sentence: "(title) by (artist) from (album)"
            if artist: desc.append(_("by %s") % artist)
            # TRANSLATORS: Part of the sentence: "(title) by (artist) from (album)"
            if album: desc.append(_("from %s") % album)

            #self.window.set_title(_("Exaile: playing %s") % title +
            #    ' ' + ' '.join(desc))
            desc_newline = '\n'.join(desc)
            self.artist_label.set_label(desc_newline)
            if self.tray_icon:
                self.tray_icon.set_tooltip(_("Playing %s") % title + '\n' +
                    desc_newline)
        else:
            #self.window.set_title(_("Exaile: playing %s") % title)
            self.artist_label.set_label("")
            if self.tray_icon:
                self.tray_icon.set_tooltip(_("Playing %s") % title)

        row = self.db.read_one("tracks, paths", "paths.name, user_rating", 
            "paths.name=? AND paths.id=tracks.path", (track.loc,))
        if row:
            rating = row[1]
            if rating <= 0 or rating == '' or rating is None: 
                rating = 0

            self.rating_combo.set_active(rating)
            track.user_rating = rating
            self.rating_combo.set_sensitive(True)
        else:
            self.rating_combo.set_active(0)
            self.rating_combo.set_sensitive(False)

        self.rating_signal = self.rating_combo.connect('changed',
            self.set_rating)

        self.emit('track-information-updated')
        if returntrue: return True

    def update_rating(self, track, plays = 1, rating = 0): 
        """
            Adds one to the "plays" of this track
        """

        update_string = "rating = rating + " + str(rating) + " , " + \
            "plays = plays + " + str(plays)

        xlmisc.log("updated plays " + str(plays) + ", rating "+ str(rating))

        path_id = library.get_column_id(self.db, 'paths', 'name', track.loc)
        self.db.execute("UPDATE tracks SET %s WHERE path=?" % update_string, 
            (path_id,))

        track.playcount += plays

        self.tracks.refresh_row(track)

    
    def setup_right(self): 
        """
            Sets up the right side of the sash (this is the playlist area)
        """
        self.cover = xlmisc.ImageWidget()
        self.cover.set_image_size(covers.COVER_WIDTH, covers.COVER_WIDTH)
        self.cover_box = covers.CoverEventBox(self, self.cover)
        self.xml.get_widget('image_box').pack_start(self.cover_box)
        self.cover.set_image(xl.path.get_data('images', 'nocover.png'))

        # set the font/etc 
        self.title_label = self.xml.get_widget('title_label')
        attr = pango.AttrList()
        attr.change(pango.AttrWeight(pango.WEIGHT_BOLD, 0, 800))
        attr.change(pango.AttrSize(12500, 0, 600))
        self.title_label.set_attributes(attr)

        burnprogs = xl.burn.check_burn_progs()
        if not burnprogs:            
            xlmisc.log("A supported CD burning program was not found "
                    "in $PATH, disabling burning capabilities.")
        else:
            pref = self.settings.get_str('burn_prog', burn.check_burn_progs()[0])

    @common.synchronized
    def new_page(self, title=_("Playlist"), songs=None, set_current=True,
        ret=True):
        """
            Create a new tab with the specified title, populates it with the
            specified songs, and sets it to be the current page if set_current
            is true.
        """
        # if there is currently only one tab, and it's an empty "Playlist"
        # tab, remove it before adding this new one
        if self.playlists_nb.get_n_pages() == 1:
            page = self.playlists_nb.get_nth_page(0)
            tab = self.playlists_nb.get_tab_label(page)
            if tab.title == _("Playlist") and self.tracks and not \
                self.tracks.songs:
                self.playlists_nb.remove_page(0)
        
        if not songs: songs = library.TrackData()

        self.tracks = trackslist.TracksListCtrl(self)
        t = self.tracks
        self.tracks.playlist_songs = songs 
        tab = xlmisc.NotebookTab(self, title, self.tracks)
        self.playlists_nb.append_page(self.tracks, tab)
        self.playlists_nb.set_tab_reorderable(self.tracks, True)
        
        if set_current:
            self.playlists_nb.set_current_page( 
                self.playlists_nb.get_n_pages() - 1)
            self.update_songs(songs)

        # if there is more than one tab, show the tab bar
        if self.playlists_nb.get_n_pages() > 1:
            self.playlists_nb.set_show_tabs(True)

        if ret: return t

    def close_page(self, page=None): 
        """
            Called when the user clicks "Close" in the notebook popup menu
        """
        nb = self.playlists_nb
        if not page:
            i = self.playlists_nb.get_current_page()
            if i > -1:
                page = self.playlists_nb.get_nth_page(i)
                page.close_page()
                self.playlists_nb.remove_page(i)
        else:
            for i in range(0, nb.get_n_pages()):
                p = nb.get_nth_page(i)
                if p == page:
                    page.close_page()
                    nb.remove_page(i)
                    break

        self.tracks = None
        
        if self.playlists_nb.get_n_pages() == 0:
            self.new_page(_("Playlist"))
            return False

        num = nb.get_current_page()
        self.page_changed(nb, None, num)
        return False

    def clear_playlist(self, *e): 
        """
            Clears the current playlist
        """

        if self.tracks == None:
            self.new_page()

        self.tracks.set_songs(library.TrackData())
        self.tracks.playlist_songs = self.tracks.songs
        self.playlist_songs = self.tracks.songs
        self.songs = self.tracks.songs
    
    def on_search(self, widget=None, event=None, custom=True): 
        """
            Called when something is typed into the filter box
        """

        keyword = unicode(self.tracks_filter.get_text(), 'utf-8')
        if keyword.startswith("where ") and not widget: return
        self.songs = library.search(self, self.tracks.playlist_songs, None,
            custom=custom)
        self.tracks.set_songs(self.songs, False)
        
        tokens = keyword.lower().split()
        for token in tokens:
            self.songs = library.search(self, self.songs, token, custom=custom)
            self.tracks.set_songs(self.songs, False)


    def __on_volume_scroll(self, widget, ev):
        """
            Called when the user scrolls their mouse wheel over the volume bar
        """
        # Modify default HScale up/down behaviour.
        if ev.direction == gtk.gdk.SCROLL_DOWN:
            self.volume.page_down()
            return True
        elif ev.direction == gtk.gdk.SCROLL_UP:
            self.volume.page_up()
            return True
        return False

    def __on_volume_key_press(self, widget, ev):
        """
            Called when the user presses a key when the volume bar is focused
        """
        # Modify default HScale up/down behaviour.
        inc = widget.get_adjustment().props.step_increment
        if ev.keyval == gtk.keysyms.Down:
            self.volume.step_down()
            return True
        elif ev.keyval == gtk.keysyms.Up:
            self.volume.step_up()
            return True
        return False

    def __on_volume_changed(self, adjustment): 
        """
            Called when the volume is changed
        """

        value = adjustment.get_value()
        frac_value = value / 100.0
        self.player.set_volume(frac_value)
        self.settings['volume'] = frac_value
        if not self.window.has_toplevel_focus() and self.settings.get_boolean("osd/enabled", True):
            pop = xlmisc.get_osd(self, xlmisc.get_osd_settings(self.settings))
            vol_text = "<big><b> " + _("Changing volume: %d %%") % \
                self.get_volume_percent() + "</b></big>"
            pop.show_osd(vol_text, None)

    def seek_begin(self, widget, event):
        """
            Starts when seek drag begins
        """
        if not self.player.current or self.player.current.type == \
            'stream': return
        self.seeking = True

    def seek_motion_notify(self, widget, event):
        """
            Simulates dragging on the new progressbar widget
        """
        if not self.player.current or self.player.current.type == \
            'stream': return
        mouse_x, mouse_y = event.get_coords()
        progress_loc = self.new_progressbar.get_allocation()

        value = mouse_x / progress_loc.width
        if value < 0: value = 0
        if value > 1: value = 1
        self.new_progressbar.set_fraction(value)
        track = self.player.current

        duration = track.duration
        if duration == -1:
            real = 0
        else:
            real = value * duration
        seconds = real

        remaining_seconds = duration - seconds
        self.new_progressbar.set_text("%d:%02d / %d:%02d" % ((seconds / 60), 
            (seconds % 60), (remaining_seconds / 60), (remaining_seconds % 60))) 

    def seek_end(self, widget, event):
        """
            Resets seeking flag, actually seeks to the requested location
        """

        mouse_x, mouse_y = event.get_coords()
        progress_loc = self.new_progressbar.get_allocation()

        value = mouse_x / progress_loc.width
        if value < 0: value = 0
        if value > 1: value = 1

        if not self.player.current or \
            self.player.current.type == 'stream':
            self.new_progressbar.set_fraction(0)
            return
        duration = self.player.current.duration * gst.SECOND
        if duration == -1:
            real = 0
        else:
            real = value * duration / 100
        seconds = real / gst.SECOND

        duration = self.player.current.duration
        real = float(value * duration)
        self.player.seek(real)
        self.seeking = False
        self.player.current.submitted = True
        self.emit('seek', real)

    def get_suggested_songs(self):
        """
            Gets suggested tracks from last.fm
        """
        if not self.tracks or not self.player.current: return

        played = 0
        for song in self.songs:
            if song in self.player.played:
                played += 1

        count = 5 - (len(self.songs) - played)
        xlmisc.log("suggested song count is %d" % count)
        if count <= 0: count = 1

        songs = library.get_suggested_songs(self, self.db, 
            self.player.current, self.songs, count, self.add_suggested)

    def add_suggested(self, artists, count):
        """
            adds suggested tracks that were fetched
        """
        songs = library.TrackData()
        for artist in artists:
            rows = self.db.select("SELECT paths.name FROM artists,tracks,paths WHERE " 
                "tracks.path=paths.id AND artists.id=tracks.artist AND "
                "artists.name=?", (unicode(artist),))
            if rows:
                search_songs = []
                for row in rows:
                    song = self.all_songs.for_path(row[0])
                    if song:
                        search_songs.append(song)

                if search_songs:
                    random.shuffle(search_songs)
                    song = search_songs[0]
                    if not song in self.tracks.songs \
                        and not song in self.player.played and not \
                        song in self.player.queued:
                        songs.append(song)

            if len(songs) >= count: break

        if not songs:
            # TRANSLATORS: For dynamic playlist
            self.status.set_first(_("Could not find any"
            " suggested songs"), 4000)

        for song in songs:
            self.tracks.append_song(song)

        self.update_songs(None, False)

    def show_osd(self, tray=False):
        """
            Shows a popup window with information about the current track
        """
        if tray:
            if not self.settings.get_boolean('osd/tray', True): return
        else:
            if not self.settings.get_boolean("osd/enabled", True): return
        track = self.player.current
        if not track: return
        pop = xlmisc.get_osd(self, xlmisc.get_osd_settings(self.settings))
        cover = self.cover_manager.fetch_cover(track, 1)

        text_display = self.settings.get_str('osd/display_text',
            xl.prefs.TEXT_VIEW_DEFAULT)
        pop.show_track_osd(track, text_display,
            cover)
        self.timer_update()

    def setup_menus(self):
        """
            Sets up menus
        """
        self.shuffle = self.xml.get_widget('shuffle_button')
        self.shuffle.set_active(self.settings.get_boolean('shuffle', False))
        self.player.shuffle = self.shuffle.get_active()
        self.shuffle.connect('toggled', self.toggle_mode, 'shuffle')

        self.repeat = self.xml.get_widget('repeat_button')
        self.repeat.set_active(self.settings.get_boolean('repeat', False))
        self.player.repeat = self.repeat.get_active()
        self.repeat.connect('toggled', self.toggle_mode, 'repeat')

        self.dynamic = self.xml.get_widget('dynamic_button')
        self.dynamic.set_active(self.settings.get_boolean('dynamic', False))
        self.dynamic.connect('toggled', self.toggle_mode, 'dynamic')

    def toggle_mode(self, item, param):
        """
            Toggles the settings for the specified playback mode
        """
        self.settings.set_boolean(param, item.get_active())
        setattr(self.player, param, item.get_active())

    
    def get_last_dir(self):
        """
            Gets the last working directory
        """

        try:
            f = self.last_open_dir
        except:
            self.last_open_dir = self.settings.get_str('last_open_dir',
                xl.path.home)
        return self.last_open_dir

    def on_add_media(self, item, event=None): 
        """
            Adds media to the current selected tab regardless of whether or
            not they are contained in the library
        """
        dialog = gtk.FileChooserDialog(_("Choose a file"), self.window,
            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OPEN, gtk.RESPONSE_OK))

        new_tab = gtk.CheckButton(_("Open in new tab"))
        dialog.set_extra_widget(new_tab)
        dialog.set_current_folder(self.get_last_dir())
        dialog.set_select_multiple(True)

        supported = gtk.FileFilter()
        supported.set_name(_("Supported Files"))
        music = gtk.FileFilter()
        music.set_name(_("Music Files"))
        playlist = gtk.FileFilter()
        playlist.set_name(_("Playlist Files"))
        all = gtk.FileFilter()
        all.set_name(_("All Files"))

        for ext in media.SUPPORTED_MEDIA:
            supported.add_pattern('*' + ext)
            music.add_pattern('*' + ext)
        for ext in xlmisc.PLAYLIST_EXTS:
            supported.add_pattern('*' + ext)
            playlist.add_pattern('*' + ext)
        all.add_pattern('*')

        dialog.add_filter(supported)
        dialog.add_filter(music)
        dialog.add_filter(playlist)
        dialog.add_filter(all)

        result = dialog.run()
        dialog.hide()

        if result == gtk.RESPONSE_OK:
            paths = dialog.get_filenames()
            dir = dialog.get_current_folder()
            if dir: # dir is None when the last view is a search
                self.last_open_dir = dir
            self.status.set_first(_("Populating playlist..."))
            songs = library.TrackData()

            count = 0
            for path in paths:
                (f, ext) = os.path.splitext(path)
                if ext in media.SUPPORTED_MEDIA:
                    if count >= 10:
                        xlmisc.finish()
                        count = 0
                    tr = library.read_track(self.db, self.all_songs, path)

                    count = count + 1
                    if tr:
                        songs.append(tr)
                if ext in xlmisc.PLAYLIST_EXTS:
                    self.playlist_manager.import_playlist(path, 
                        newtab=new_tab.get_active())

            if songs:
                if new_tab.get_active():
                    self.new_page(_("Playlist"), songs)
                else:
                    self.playlist_manager.append_songs(songs)

            self.status.set_first(None)

    def get_volume_percent(self):
        """
            Returns the current volume level as a percentage
        """
        vol = self.volume.get_value()
        return round(vol)

    def stream(self, url): 
        """
            Play a radio stream
        """
        self.player.stop()

        # plugins can register as urlhandlers, so check to see if any plugin
        # wants to handle this url
        for handler in self.urlhandlers:
            if hasattr(handler, 'handles_url') and \
                hasattr(handler, 'handle_url'):
                if handler.handles_url(url):
                    handler.handle_url(url)
                    return

        if "://" in url:
            track = media.Track(url)
            track.type = 'stream'
        else:
            lowurl = url.lower()
            # if it's a playlist file
            if common.any(lowurl.endswith(ext) for ext in xlmisc.PLAYLIST_EXTS):
                self.playlist_manager.import_playlist(url, True)
                return
            else:
                track = library.read_track(self.db, self.all_songs, url)

        songs = library.TrackData((track, ))
        if not songs: return

        self.playlist_manager.append_songs(songs, play=False)
        self.player.play_track(track)

    def open_url(self, event): 
        """
            Prompts for a url to open
        """
        dialog = common.TextEntryDialog(self.window,
            _("Enter the address"), _("Enter the address"))
        result = dialog.run()

        if result == gtk.RESPONSE_OK:
            path = dialog.get_value()
            self.stream(path)

    def open_disc(self, widget=None):
        """
            Opens an audio disc (only one tab allowed)
        """
        if not library.CDDB_AVAIL:
            common.error(self.window, _('You need the python-cddb package '
                'in order to play audio discs.'))
            return

        for i in range(self.playlists_nb.get_n_pages()):
            if self.playlists_nb.get_nth_page(i) == self.audio_disc_page:
                self.playlists_nb.set_current_page(i)
                return

        songs = library.read_audio_disc(self)
        if not songs: return
        self.audio_disc_page = self.new_page(_("Audio Disc"), songs)

    def goto_current(self, *e): 
        """
            Ensures that the currently playing track is visible
        """
        if not self.tracks: return
        self.tracks.ensure_visible(self.player.current)

    def on_sigterm(self, signalnum, stackframe):
        xlmisc.log("Caught SIGTERM, cleaning up.")
        gobject.idle_add(self.on_quit)

    def on_quit(self, widget=None, event=None): 
        """
            Saves the current playlist and exits.  If user closes the window
            while tray icon is present, simply hides the window.
        """
        self.window.hide()
        xlmisc.finish()
        if self.tray_icon and widget == self.window:
            return True

        # PLUGIN: send plugins event before quitting
        self.emit('quit')

        # Write any tracks remaining in the last.fm cache to disk
        # for submission later.
        self.scrobbler_write_cache()

        self.player.stop()
        self.cover_manager.stop_cover_thread()
        for thread in self.thread_pool:
            thread.done = True

        dir = xl.path.get_config('saved')
        if not os.path.isdir(dir):
            os.mkdir(dir)

        # delete all current saved playlists
        for file in os.listdir(dir):
            if file.endswith(".m3u"):
                os.unlink(os.path.join(dir, file))

        queuefile = xl.path.get_config('queued.save')
        if os.path.isfile(queuefile):
            os.unlink(queuefile)
            

        if self.player.current: self.player.current.stop()
        # Clear search filter if needed so that the entire playlist is saved
        if self.tracks_filter.get_text() != '':
            self.tracks_filter.set_text('')
            try:
                self.on_search()
            except:  # In case we're quitting before the playlist loaded
                pass

        for i in range(self.playlists_nb.get_n_pages()):
            page = self.playlists_nb.get_nth_page(i)
            title = self.playlists_nb.get_tab_label(page).title
            if page.type != 'track' or page == self.audio_disc_page: continue
            songs = page.songs
            self.playlist_manager.save_m3u(xl.path.get_config('saved',
                "playlist%.4d.m3u" % i), songs, title)

        # save queued tracks
        if self.player.queued:
            h = open(os.path.join(dir, "queued.save"), "w")
            for song in self.player.queued:
                h.write("%s\n" % song.loc)
            h.close()
        elif os.path.isfile(os.path.join(dir, "queued.save")):
            os.unlink(os.path.join(dir, "queued.save"))

        if self.player.stop_track:
            self.settings.set_str('stop_track', self.player.stop_track.loc)
        else:
            self.settings['stop_track'] = ''

        self.db.db.commit()
        last_active = self.playlists_nb.get_current_page()
        xlmisc.log('Last active is: %d' % last_active)
        self.settings['last_active'] = last_active
        self.settings.save()
        
        gtk.main_quit()
        print 'Exiting, bye!'

    def on_resize(self, widget, event): 
        """
            Saves the current size and position
        """
        if self.settings.get_boolean('ui/mainw_maximized', False): return False

        (width, height) = self.window.get_size()
        self.settings['ui/mainw_width'] = width
        self.settings['ui/mainw_height'] = height
        (x, y) = self.window.get_position()
        self.settings['ui/mainw_x'] = x
        self.settings['ui/mainw_y'] = y
        if self.splitter.get_position() > 10:
            sash = self.splitter.get_position()
            self.settings['ui/mainw_sash_pos'] = sash
        return False

    def on_state_change(self, widget, event):
        """
            Saves the current maximized state
        """
        if event.changed_mask & gtk.gdk.WINDOW_STATE_MAXIMIZED:
            self.settings.set_boolean('ui/mainw_maximized',
                bool(event.new_window_state & gtk.gdk.WINDOW_STATE_MAXIMIZED))
        return False

    def sync_playlists_tabbar(self, *args):
        """
            Hides/unhides the tab bar according to the number of pages
        """
        self.playlists_nb.set_show_tabs(
            self.settings.get_boolean('ui/always_show_tabbar', True)
            or self.playlists_nb.get_n_pages() > 1)

    def jump_to(self, index):
        """
            Show the a specific page in the track information tab about
            the current track
        """
        track = self.player.current
        if not track and not self.tracks: return
        if not track: track = self.tracks.get_selected_track() 
        if not track: return
            
        page = information.show_information(self, track)
        page.set_current_page(index)



"""
	Add Directory Plugin
	
	With this plugin you can select folders instead of files to add
	to your playlist.
	
	Created By Lucas van Dijk <info@return.net> (http://www.retun1.net)
"""
 
import gtk
from xl import xlmisc
from xl import media, library
import xl.plugins as plugins
from gettext import gettext as _
import os, os.path
 
PLUGIN_NAME = _("Add Directory")
PLUGIN_AUTHORS = ['Lucas van Dijk <info@return1.net> (http://www.return1.net)']
PLUGIN_VERSION = '0.1'
PLUGIN_DESCRIPTION = _("With this plugin you can select folders instead of files to add to your playlist.")
PLUGIN_ENABLED = False
 
b = gtk.Button()
PLUGIN_ICON = b.render_icon(gtk.STOCK_OPEN, gtk.ICON_SIZE_MENU)
b.destroy()
 
def initialize():
	"""
		Called when the plugin is enabled
	"""
	
	# Add Menu entry
	try:
		menu_item = gtk.MenuItem(_('Open Directory'))	
		menu_item.connect('activate', on_add_dir)
		menu_item.set_name('add_dir_plugin_item')
		
		menubar = APP.xml.get_widget('file_menu_menu')	
		menubar.insert(menu_item, 3)
		
		menu_item.show()
	except Exception, e:
		print e
	return True
 
def destroy():
	"""
		Called when the plugin is disabled
	"""
	menubar = APP.xml.get_widget('file_menu_menu')
	for child in menubar.get_children():
		if child.get_name() == 'add_dir_plugin_item':
			menubar.remove(child)
			
class TrackDataEx(library.TrackData):
	"""
		This subclass allows merging of 2 TrackData instances
	"""
	
	def append(self, tracks):
		if isinstance(tracks, library.TrackData):
			for track in tracks:
				library.TrackData.append(self, track)
		else:
			library.TrackData.append(self, tracks)
		
 
def on_add_dir(item, event = None):	
	dialog = gtk.FileChooserDialog(_("Choose directories"), APP.window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		
	hbox = gtk.HBox(True)
	
	new_tab = gtk.CheckButton(_("Open in new tab"))
	recurse = gtk.CheckButton(_("Recursive"))
	recurse.set_active(True)
	
	hbox.pack_start(new_tab)
	hbox.pack_start(recurse)
	hbox.show_all()
	
	dialog.set_extra_widget(hbox)
	dialog.set_current_folder(APP.get_last_dir())
	dialog.set_select_multiple(True)

	music = gtk.FileFilter()
	music.set_name(_("Music Files"))
	all = gtk.FileFilter()
	all.set_name(_("All Files"))

	for ext in media.SUPPORTED_MEDIA:
		music.add_pattern('*' + ext)
		
	all.add_pattern('*')

	dialog.add_filter(music)
	dialog.add_filter(all)

	result = dialog.run()
	dialog.hide()
	
	if result == gtk.RESPONSE_OK:
		paths = dialog.get_filenames()
		dir = dialog.get_current_folder()
		if dir: # dir is None when the last view is a search
			APP.last_open_dir = dir
		APP.status.set_first(_("Populating playlist..."))	
		
		songs = TrackDataEx()
		for dir in paths:
			songs_to_append = import_dir(dir, new_tab.get_active(), recurse.get_active())
			
			if songs_to_append:
				songs.append(songs_to_append)
		
		if songs:
			if new_tab.get_active():
				APP.new_page(_("Playlist"), songs)
			else:
				APP.playlist_manager.append_songs(songs)
				
def import_dir(dir, new_tab, recurse):
	songs = TrackDataEx()
	count = 0
	
	for file in os.listdir(dir):
		path = os.path.join(dir, file)
		
		if os.path.isdir(path):
			if recurse:
				songs_to_append = import_dir(path, new_tab, recurse)
				if songs_to_append:
					songs.append(songs_to_append)
		else:
			(f, ext) = os.path.splitext(path)
			if ext in media.SUPPORTED_MEDIA:
				if count >= 10:
					xlmisc.finish()
					count = 0
				tr = library.read_track(APP.db, APP.all_songs, path)

				count += 1
				if tr:
					songs.append(tr)	
	return songs



# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk, subprocess, os, time, gst
from gettext import gettext as _
from xl import common, media, xlmisc
import xl.plugins as plugins
import xl.path

PLUGIN_NAME = _("Streamripper!")
PLUGIN_AUTHORS = ['Adam Olsen <arolsen@gmail.com>']
PLUGIN_VERSION = '0.1.4'
PLUGIN_DESCRIPTION = _(r"""Allows you to record streams with
streamripper\n\nRequires the command line version of streamripper to be
installed""")

PLUGIN_ENABLED = False
button = gtk.Button()
PLUGIN_ICON = button.render_icon('gtk-media-record', gtk.ICON_SIZE_MENU)
button.destroy()
PLUGIN = None
APP = None
BUTTON = None
STREAMRIPPER_PID = None
STREAMRIPPER_OUT = None
CURRENT_TRACK = None
CONS = plugins.SignalContainer()

def configure():
    """
        Shows the configuration dialog for streamripper
    """
    exaile = APP
    dialog = plugins.PluginConfigDialog(exaile.window, PLUGIN_NAME)
    table = gtk.Table(2, 2)
    table.set_row_spacings(2)
    bottom = 0
    label = gtk.Label(_("Save Location:    "))
    label.set_alignment(0.0, 0.0)

    table.attach(label, 0, 1, bottom, bottom + 1)

    location = exaile.settings.get_str("save_location", plugin=plugins.name(__file__),
        default=os.getenv("HOME"))
    save_loc = gtk.FileChooserButton(_("Location"))
    save_loc.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
    save_loc.set_current_folder(location)

    table.attach(save_loc, 1, 2, bottom, bottom + 1)
    bottom += 1

    label = gtk.Label(_("Relay Port:"))
    label.set_alignment(0.0, 0.0)
    table.attach(label, 0, 1, bottom, bottom + 1)

    port = exaile.settings.get_str("relay_port", plugin=plugins.name(__file__), default='8000')
    port_entry = gtk.Entry()
    port_entry.set_text(port)
    port_entry.set_max_length(6)
    table.attach(port_entry, 1, 2, bottom, bottom + 1, gtk.SHRINK)

    dialog.child.pack_start(table)
    dialog.show_all()

    result = dialog.run()
    dialog.hide()
    if result == gtk.RESPONSE_OK:
        exaile.settings.set_str("save_location", save_loc.get_current_folder(),
            plugin=plugins.name(__file__))
        exaile.settings.set_str("relay_port", port_entry.get_text(),
            plugin=plugins.name(__file__))

def toggle_record(widget, event=None):
    """
        Toggles streamripper
    """
    global STREAMRIPPER_PID, STREAMRIPPER_OUT, CURRENT_TRACK
    track = APP.player.current
    if not STREAMRIPPER_PID:
        if not track: return True
        if not track.type == 'stream':
            common.error(APP.window, _("You can only record streams"))
            widget.set_active(False)
            return True

        savedir = SETTINGS.get_str('save_location',
            plugin=plugins.name(__file__),
            default=os.getenv("HOME"))
        port = SETTINGS.get_int('relay_port', plugin=plugins.name(__file__),
            default=8000)
        outfile = xl.path.get_config('streamripper.log')

        STREAMRIPPER_OUT = open(outfile, "w+", 0)
        STREAMRIPPER_OUT.write("Streamripper log file started: %s\n" %
            time.strftime("%c", time.localtime()))
        STREAMRIPPER_OUT.write(
            "-------------------------------------------------\n\n\n")

        APP.player.playbin.set_state(gst.STATE_NULL)
        xlmisc.log('real location: %s' %
            APP.player.playbin.get_property('uri'))
        sub = subprocess.Popen(['streamripper',
            APP.player.playbin.get_property('uri'), '-r',
            str(port), '-d', savedir], stderr=STREAMRIPPER_OUT)
        ret = sub.poll()

        xlmisc.log("Streamripper return value was %s" % ret)
        xlmisc.log("Using streamripper to play location: %s" % track.loc)
        APP.status.set_first("Streamripping location: %s..." %
            track.loc, 4000)
        if ret != None:
            common.error(APP.window, _("There was an error"
                " executing streamripper."))
            return True
        STREAMRIPPER_PID = sub.pid

        xlmisc.log('Proxy location: http://localhost:%d' % port)
        APP.player.playbin.set_property('uri', "http://localhost:%d" % port)
        time.sleep(1)
        APP.player.playbin.set_state(gst.STATE_PLAYING)
        CURRENT_TRACK = track

        return False
    else:
        if not STREAMRIPPER_PID:
            common.error(APP.window, _("Streamripper is not running."))
        os.system("kill -9 %d" % STREAMRIPPER_PID)
        APP.player.playbin.set_state(gst.STATE_READY)
        CURRENT_TRACK = None
        APP.player.play_track(track)
        STREAMRIPPER_PID = None

    return False

def stop():
    """
        Stops streamripper by killing it if it's still running, and closes the
        log file
    """
    global STREAMRIPPER_OUT, STREAMRIPPER_PID
    if BUTTON:  
        BUTTON.set_active(False)
    if STREAMRIPPER_OUT:
        try:
            STREAMRIPPER_OUT.close()
        except OSError:
            pass
        STREAMRIPPER_OUT = None

    if STREAMRIPPER_PID:
        os.system("kill -9 %d" % STREAMRIPPER_PID)
        STREAMRIPPER_PID = None
    

def stop_track(exaile, track):
    """
        Called when playback has stopped on a track
    """
    stop()

def initialize():
    """
        Checks for streamripper, initializes the plugin
    """
    global APP, SETTINGS, BUTTON
    exaile = APP
    try:
        subprocess.call(['streamripper'], stdout=-1, stderr=-1)
    except OSError:
        raise plugins.PluginInitException("Streamripper is not available, "
            "disabling streamripper plugin")
        return False

    APP = exaile
    SETTINGS = exaile.settings
    BUTTON = gtk.ToggleButton()
    BUTTON.connect('button-release-event', toggle_record)
    image = gtk.Image()
    image.set_from_stock('gtk-media-record', gtk.ICON_SIZE_SMALL_TOOLBAR)
    BUTTON.set_image(image)

    toolbar = APP.xml.get_widget('play_toolbar')
    toolbar.pack_start(BUTTON, False, False)
    toolbar.reorder_child(BUTTON, 3)

    CONS.connect(APP.player, 'stop-track', stop_track)
    BUTTON.show()

    return True

def destroy():
    """
        Called when the plugin is disabled.  If streamripper is currently
        running, kill it and restart the track without using streamripper
    """
    global BUTTON, CURRENT_TRACK
    stop()
    if CURRENT_TRACK:
        CURRENT_TRACK.stream_url = None
        if CURRENT_TRACK.is_playing():
            CURRENT_TRACK.stop()
            CURRENT_TRACK.play(APP.on_next)
        CURRENT_TRACK = None
    if not BUTTON: return
    toolbar = APP.xml.get_widget('play_toolbar')
    toolbar.remove(BUTTON)
    BUTTON.hide()
    BUTTON.destroy()
    BUTTON = None
    CONS.disconnect_all()



# desktopcover - displays Exaile album covers on the desktop
# Copyright (C) 2006-2007 Johannes Sasongko <sasongko@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import gobject, gtk
from gettext import gettext as _
from xl import xlmisc
import xl.plugins as plugins

PLUGIN_NAME = _("Desktop Cover")
PLUGIN_AUTHORS = ["Johannes Sasongko <sasongko@gmail.com>", 
    "Adam Olsen <arolsen@gmail.com>"]

PLUGIN_VERSION = "0.3.3"
PLUGIN_DESCRIPTION = _("Displays the current album cover on the desktop")
PLUGIN_ENABLED = False
PLUGIN_ICON = None

PLUGIN = None
CONNS = plugins.SignalContainer()

class CoverDisplay:
    DEFAULT_X = DEFAULT_Y = 0
    DEFAULT_WIDTH = DEFAULT_HEIGHT = 200
    GRAVITY_SIGNS = {
        gtk.gdk.GRAVITY_NORTH_WEST: ('+', '+'),
        gtk.gdk.GRAVITY_NORTH_EAST: ('-', '+'),
        gtk.gdk.GRAVITY_SOUTH_WEST: ('+', '-'),
        gtk.gdk.GRAVITY_SOUTH_EAST: ('-', '-'),
    }

    def __init__(self):
        self.window = wnd = gtk.Window()
        wnd.set_accept_focus(False)
        wnd.set_decorated(False)
        #wnd.set_keep_below(True)
        wnd.set_resizable(False)
        wnd.set_role(__file__)
        wnd.set_skip_pager_hint(True)
        wnd.set_skip_taskbar_hint(True)
        wnd.set_title("")
        #wnd.stick()

        self.x = self.DEFAULT_X
        self.y = self.DEFAULT_Y
        self.width = self.DEFAULT_WIDTH
        self.height = self.DEFAULT_HEIGHT
        self.cover = None
        self.set_use_image_size()
        self.set_keep_center()

        self.image = img = gtk.Image()
        wnd.add(img)
        img.show()

    def set_keep_center(self, keep_center=True):
        self.keep_center = keep_center
        xlmisc.finish()
        if keep_center:
            self.window.set_position(gtk.WIN_POS_CENTER_ALWAYS)
        else:
            self.window.set_position(gtk.WIN_POS_NONE)
            self.set_position()

    def set_gravity(self, gravity):
        xlmisc.finish()
        self.window.set_gravity(gravity)

    def set_position(self, x=None, y=None, force=False):
        if x is None or y is None:
            x = self.x
            y = self.y
        else:
            self.x = x
            self.y = y

        xlmisc.finish()
        if not self.keep_center and (force or self.window.props.visible):
            xsgn, ysgn = self.GRAVITY_SIGNS[self.window.get_gravity()]
            xlmisc.finish()
            self.window.parse_geometry('%s%s%s%s' % (xsgn, x, ysgn, y))

    def set_use_image_size(self, use_image_size=True):
        self.use_image_size = use_image_size

    def set_size(self, width, height):
        self.width = width
        self.height = height
        self.display(self.cover)

    def display(self, cover):
        self.cover = cover
        if cover is None:
            self.image.clear()
            xlmisc.finish()
            self.window.hide()
            return

        pixbuf = gtk.gdk.pixbuf_new_from_file(cover)
        width = pixbuf.get_width()
        height = pixbuf.get_height()
        if not self.use_image_size:
            origw = float(width)
            origh = float(height)
            width, height = self.width, self.height
            scale = min(width / origw, height / origh)
            width = int(origw * scale)
            height = int(origh * scale)
            pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
        self.image.set_from_pixbuf(pixbuf)

        wnd = self.window
        self.set_position(force=True)
        xlmisc.finish()
        if not wnd.props.visible:
            wnd.show()
            # May be reset by the WM.
            wnd.set_keep_below(True)
            wnd.stick()

    def destroy(self):
        self.window.destroy()

GRAVITIES = [
    # TRANSLATORS: Desktop cover gravity
    (_("Northwest"), gtk.gdk.GRAVITY_NORTH_WEST),
    # TRANSLATORS: Desktop cover gravity
    (_("Northeast"), gtk.gdk.GRAVITY_NORTH_EAST),
    # TRANSLATORS: Desktop cover gravity
    (_("Southwest"), gtk.gdk.GRAVITY_SOUTH_WEST),
    # TRANSLATORS: Desktop cover gravity
    (_("Southeast"), gtk.gdk.GRAVITY_SOUTH_EAST),
]

class DesktopCoverConfig(plugins.PluginConfigDialog):
    def __init__(self, exaile, title, plugin, plugin_name):
        super(type(self), self).__init__(exaile.window, title)
        self.exaile = exaile
        settings = exaile.settings
        self.plugin = plugin
        self.plugin_name = plugin_name

        table = gtk.Table(6, 2)
        table.set_border_width(12)
        table.set_col_spacings(6)
        self.main.add(table)

        self.position_widgets = position_widgets = []
        self.size_widgets = size_widgets = []

        n_rows = 0

        self.position_check = check = gtk.CheckButton(_("Manual positioning"))
        table.attach(check, 0, 2, n_rows, n_rows + 1)
        check.connect('toggled', self._positioning_toggled)
        check.set_active(not settings.get_boolean('keep_center',
            default=True, plugin=plugin_name))
        n_rows += 1

        label = gtk.Label(_("Gravity"))
        table.attach(label, 0, 1, n_rows, n_rows + 1)
        position_widgets.append(label)
        self.gravity_combo = combo = gtk.combo_box_new_text()
        table.attach(combo, 1, 2, n_rows, n_rows + 1)
        position_widgets.append(combo)
        for grav in GRAVITIES:
            combo.append_text(grav[0])
        combo.set_active(settings.get_int('gravity', default=0,
            plugin=plugin_name))
        n_rows += 1

        # TRANSLATORS: Offset from the desktop border
        label = gtk.Label(_("X offset"))
        table.attach(label, 0, 1, n_rows, n_rows + 1)
        position_widgets.append(label)
        x = settings.get_int('x', default=CoverDisplay.DEFAULT_X,
            plugin=plugin_name)
        adj = gtk.Adjustment(x, 0, 32767, 1, 10, 10)
        self.x_spin = spin = gtk.SpinButton(adj)
        table.attach(spin, 1, 2, n_rows, n_rows + 1)
        position_widgets.append(spin)
        n_rows += 1

        # TRANSLATORS: Offset from the desktop border
        label = gtk.Label(_("Y offset"))
        table.attach(label, 0, 1, n_rows, n_rows + 1)
        position_widgets.append(label)
        y = settings.get_int('y', default=CoverDisplay.DEFAULT_Y,
            plugin=plugin_name)
        adj = gtk.Adjustment(y, 0, 32767, 1, 10, 10)
        self.y_spin = spin = gtk.SpinButton(adj)
        position_widgets.append(spin)
        table.attach(spin, 1, 2, n_rows, n_rows + 1)
        n_rows += 1

        self.sizing_check = check = gtk.CheckButton(_("Manual sizing"))
        table.attach(check, 0, 2, n_rows, n_rows + 1)
        check.connect('toggled', self._sizing_toggled)
        check.set_active(not settings.get_boolean('use_image_size',
            default=True, plugin=plugin_name))
        n_rows += 1

        label = gtk.Label(_("Width"))
        table.attach(label, 0, 1, n_rows, n_rows + 1)
        size_widgets.append(label)
        w = settings.get_int('width', default=CoverDisplay.DEFAULT_WIDTH,
            plugin=plugin_name)
        adj = gtk.Adjustment(w, 0, 32767, 1, 10, 10)
        self.width_spin = spin = gtk.SpinButton(adj)
        table.attach(spin, 1, 2, n_rows, n_rows + 1)
        size_widgets.append(spin)
        n_rows += 1

        label = gtk.Label(_("Height"))
        table.attach(label, 0, 1, n_rows, n_rows + 1)
        size_widgets.append(label)
        h = settings.get_int('height', default=CoverDisplay.DEFAULT_HEIGHT,
            plugin=plugin_name)
        adj = gtk.Adjustment(h, 0, 32767, 1, 10, 10)
        self.height_spin = spin = gtk.SpinButton(adj)
        table.attach(spin, 1, 2, n_rows, n_rows + 1)
        size_widgets.append(spin)
        n_rows += 1

        self._setup_position_widgets()
        self._setup_size_widgets()
        table.show_all()

    def run(self):
        response = super(type(self), self).run()
        if response != gtk.RESPONSE_OK: return response

        settings = self.exaile.settings
        plugin = self.plugin
        plugin_name = self.plugin_name

        keep_center = not self.position_check.get_active()
        if plugin:
            plugin.set_keep_center(keep_center)
        settings.set_boolean('keep_center', keep_center, plugin=plugin_name)

        gravity = self.gravity_combo.get_active()
        if plugin:
            plugin.set_gravity(GRAVITIES[gravity][1])
        settings.set_int('gravity', gravity, plugin=plugin_name)

        use_image_size = not self.sizing_check.get_active()
        if plugin:
            plugin.set_use_image_size(use_image_size)
        settings.set_boolean('use_image_size', use_image_size,
            plugin=plugin_name)

        width = self.width_spin.get_value_as_int()
        height = self.height_spin.get_value_as_int()
        if plugin:
            plugin.set_size(width, height)
        settings.set_int('width', width, plugin=plugin_name)
        settings.set_int('height', height, plugin=plugin_name)

        x = self.x_spin.get_value_as_int()
        y = self.y_spin.get_value_as_int()
        if plugin:
            plugin.set_position(x, y)
        settings.set_int('x', x, plugin=plugin_name)
        settings.set_int('y', y, plugin=plugin_name)

        return response

    def _positioning_toggled(self, check, *data):
        self._setup_position_widgets(check.get_active())

    def _sizing_toggled(self, check, *data):
        self._setup_size_widgets(check.get_active())

    def _setup_position_widgets(self, enabled=None):
        if enabled is None:
            enabled = not self.exaile.settings.get_boolean('keep_center',
                default=True, plugin=self.plugin_name)
        for w in self.position_widgets:
            w.set_sensitive(enabled)

    def _setup_size_widgets(self, enabled=None):
        if enabled is None:
            enabled = not self.exaile.settings.get_boolean('use_image_size',
                default=True, plugin=self.plugin_name)
        for w in self.size_widgets:
            w.set_sensitive(enabled)

def initialize():
    global PLUGIN
    PLUGIN = CoverDisplay()

    settings = APP.settings
    plugin_name = plugins.name(__file__)

    PLUGIN.set_keep_center(settings.get_boolean('keep_center', default=True,
        plugin=plugin_name))
    gravity = settings.get_int('gravity', default=0, plugin=plugin_name)
    PLUGIN.set_gravity(GRAVITIES[gravity][1])
    x = settings.get_int('x', default=PLUGIN.DEFAULT_X, plugin=plugin_name)
    y = settings.get_int('y', default=PLUGIN.DEFAULT_Y, plugin=plugin_name)
    PLUGIN.set_position(x, y)
    PLUGIN.set_use_image_size(settings.get_boolean('use_image_size',
        default=True, plugin=plugin_name))
    width = settings.get_int('width', default=PLUGIN.DEFAULT_WIDTH,
        plugin=plugin_name)
    height = settings.get_int('height', default=PLUGIN.DEFAULT_HEIGHT,
        plugin=plugin_name)
    PLUGIN.set_size(width, height)

    player = APP.player
    if player.current and (player.is_playing() or player.is_paused()):
        _display(APP.cover.loc)

    CONNS.connect(APP.cover, 'image-changed', lambda w, c: _display(c))

    return True

def destroy():
    global PLUGIN
    CONNS.disconnect_all()
    PLUGIN.destroy()
    PLUGIN = None

def configure():
    dialog = DesktopCoverConfig(APP, PLUGIN_NAME, PLUGIN,
        plugins.name(__file__))
    dialog.run()
    dialog.destroy()

STOPPED = None

def _display(cover):
    global STOPPED
    if 'nocover' in cover:
        STOPPED = True
        # Wait to make sure playback is really stopped.
        gobject.timeout_add(200, _set_no_cover)
    else:
        STOPPED = False
        PLUGIN.display(cover)

def _set_no_cover():
    if STOPPED:
        PLUGIN.display(None)
    return False # Stop GLib timeout.

# vi: et sts=4 sw=4 tw=80
